@@ -2,7 +2,7 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2014 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2013, 2016, 2021 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2016, 2020-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *
@@ -22,16 +22,19 @@
 
 #pragma once
 
-struct ParseStack
+typedef std::unique_ptr<FILE, int(*)(FILE*)> FilePtr;
+
+struct ParseStack final
 {
 	std::vector<std::string> reading;
 	insp::flat_map<std::string, std::string, irc::insensitive_swo> vars;
-	ConfigDataHash& output;
-	ConfigFileCache& FilesOutput;
+	ServerConfig::TagMap& output;
+	ServerConfig::FileSource& FilesOutput;
 	std::stringstream& errstr;
 
 	ParseStack(ServerConfig* conf);
 	bool ParseFile(const std::string& name, int flags, const std::string& mandatory_tag = std::string(), bool isexec = false);
-	void DoInclude(ConfigTag* includeTag, int flags);
+	void DoInclude(const std::shared_ptr<ConfigTag>& includeTag, int flags);
 	void DoReadFile(const std::string& key, const std::string& file, int flags, bool exec);
+	static FilePtr DoOpenFile(const std::string& name, bool isexec);
 };
