@@ -2,11 +2,11 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2013-2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012-2013, 2017-2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2012-2013, 2017-2024 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2008 Thomas Stagner <aquanight@inspircd.org>
- *   Copyright (C) 2007-2008 Robin Burchell <robin+git@viroteck.net>
+ *   Copyright (C) 2008 Thomas Stagner <aquanight@gmail.com>
+ *   Copyright (C) 2007 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2006-2008 Craig Edwards <brain@inspircd.org>
  *
@@ -41,15 +41,6 @@ enum ModeType
 	MODETYPE_CHANNEL = 1
 };
 
-/**
- * Holds mode actions - modes can be allowed or denied.
- */
-enum ModeAction
-{
-	MODEACTION_DENY = 0, /* Drop the mode change, AND a parameter if its a parameterized mode */
-	MODEACTION_ALLOW = 1 /* Allow the mode */
-};
-
 /**
  * These fixed values can be used to proportionally compare module-defined prefixes to known values.
  * For example, if your module queries a Channel, and is told that user 'joebloggs' has the prefix
@@ -89,16 +80,21 @@ class ParamModeBase;
  * set by a user, the virtual function OnModeChange is
  * called. If you specify a value greater than 0 for
  * parameters_on or parameters_off, then when the mode is
- * set or unset respectively, std::string &parameter will
+ * set or unset respectively, std::string& parameter will
  * contain the parameter given by the user, else it will
  * contain an empty string. You may alter this parameter
  * string, and if you alter it to an empty string, and your
  * mode is expected to have a parameter, then this is
- * equivalent to returning MODEACTION_DENY.
+ * equivalent to returning false.
  */
-class CoreExport ModeHandler : public ServiceProvider
+class CoreExport ModeHandler
+	: public ServiceProvider
 {
- public:
+public:
+	/** The underlying type of a mode rank. */
+	typedef unsigned long Rank;
+
+	/** The underlying type of a mode id. */
 	typedef size_t Id;
 
 	enum Class
@@ -109,12 +105,12 @@ class CoreExport ModeHandler : public ServiceProvider
 		MC_OTHER
 	};
 
- private:
+private:
 	/** The opaque id of this mode assigned by the mode parser
 	 */
 	Id modeid;
 
- protected:
+protected:
 	/** What kind of parameters does the mode take?
 	 */
 	ParamSpec parameters_taken;
@@ -128,7 +124,7 @@ class CoreExport ModeHandler : public ServiceProvider
 	 * True if the mode requires oper status
 	 * to set.
 	 */
-	bool oper;
+	bool oper = false;
 
 	/**
 	 * Mode is a 'list' mode. The behaviour
@@ -140,7 +136,7 @@ class CoreExport ModeHandler : public ServiceProvider
 	 * whether your module can produce 'lists' or not
 	 * (e.g. banlists, etc)
 	 */
-	bool list;
+	bool list = false;
 
 	/**
 	 * The mode type, either MODETYPE_USER or
@@ -153,15 +149,15 @@ class CoreExport ModeHandler : public ServiceProvider
 	const Class type_id;
 
 	/** The prefix rank required to set this mode on channels. */
-	unsigned int ranktoset;
+	Rank ranktoset = HALFOP_VALUE;
 
 	/** The prefix rank required to unset this mode on channels. */
-	unsigned int ranktounset;
+	Rank ranktounset = HALFOP_VALUE;
 
 	/** If non-empty then the syntax of the parameter for this mode. */
 	std::string syntax;
 
- public:
+public:
 	/**
 	 * The constructor for ModeHandler initializes the mode handler.
 	 * The constructor of any class you derive from ModeHandler should
@@ -174,12 +170,11 @@ class CoreExport ModeHandler : public ServiceProvider
 	 * @param mclass The object type of this mode handler, one of ModeHandler::Class
 	 */
 	ModeHandler(Module* me, const std::string& name, char modeletter, ParamSpec params, ModeType type, Class mclass = MC_OTHER);
-	CullResult cull() CXX11_OVERRIDE;
-	virtual ~ModeHandler();
+	Cullable::Result Cull() override;
 
 	/** Register this object in the ModeParser
 	 */
-	void RegisterService() CXX11_OVERRIDE;
+	void RegisterService() override;
 
 	/**
 	 * Returns true if the mode is a list mode
@@ -254,26 +249,20 @@ class CoreExport ModeHandler : public ServiceProvider
 	/**
 	 * Called when a channel mode change access check for your mode occurs.
 	 * @param source Contains the user setting the mode.
-	 * @param channel contains the destination channel the modes are being set on.
-	 * @param parameter The parameter for your mode. This is modifiable.
-	 * @param adding This value is true when the mode is being set, or false when it is being unset.
-	 * @return allow, deny, or passthru to check against the required level
+	 * @param channel The destination channel the modes are being set on.
+	 * @param change Information regarding the mode change.
 	 */
-	virtual ModResult AccessCheck(User* source, Channel* channel, std::string &parameter, bool adding);
+	virtual ModResult AccessCheck(User* source, Channel* channel, Modes::Change& change);
 
 	/**
 	 * Called when a mode change for your mode occurs.
 	 * @param source Contains the user setting the mode.
 	 * @param dest For usermodes, contains the destination user the mode is being set on. For channelmodes, this is an undefined value.
 	 * @param channel For channel modes, contains the destination channel the modes are being set on. For usermodes, this is an undefined value.
-	 * @param parameter The parameter for your mode, if you indicated that your mode requires a parameter when being set or unset. Note that
-	 * if you alter this value, the new value becomes the one displayed and send out to the network, also, if you set this to an empty string
-	 * but you specified your mode REQUIRES a parameter, this is equivalent to returning MODEACTION_DENY and will prevent the mode from being
-	 * displayed.
-	 * @param adding This value is true when the mode is being set, or false when it is being unset.
-	 * @return MODEACTION_ALLOW to allow the mode, or MODEACTION_DENY to prevent the mode, also see the description of 'parameter'.
+	 * @param change Information regarding the mode change.
+	 * @return true to allow the mode, or false to prevent the mode, also see the description of 'parameter'.
 	 */
-	virtual ModeAction OnModeChange(User* source, User* dest, Channel* channel, std::string &parameter, bool adding); /* Can change the mode parameter as its a ref */
+	virtual bool OnModeChange(User* source, User* dest, Channel* channel, Modes::Change& change);
 
 	/**
 	 * If your mode is a listmode, then this method will be called for displaying an item list, e.g. on MODE \#channel +modechar
@@ -299,7 +288,6 @@ class CoreExport ModeHandler : public ServiceProvider
 	 */
 	virtual void OnParameterInvalid(User* user, Channel* targetchannel, User* targetuser, const std::string& parameter);
 
-
 	/**
 	 * If your mode is a listmode, this method will be called to display an empty list (just the end of list numeric)
 	 * @param user The user issuing the command
@@ -317,7 +305,7 @@ class CoreExport ModeHandler : public ServiceProvider
 	 * @param channel The channel we are checking against
 	 * @return True if the other side wins the merge, false if we win the merge for this mode.
 	 */
-	virtual bool ResolveModeConflict(std::string &their_param, const std::string &our_param, Channel* channel);
+	virtual bool ResolveModeConflict(const std::string& their_param, const std::string& our_param, Channel* channel);
 
 	/**
 	 * When a MODETYPE_USER mode handler is being removed, the core will call this method for every user on the server.
@@ -341,13 +329,10 @@ class CoreExport ModeHandler : public ServiceProvider
 	/** Retrieves the level required to modify this mode.
 	 * @param adding Whether the mode is being added or removed.
 	 */
-	inline unsigned int GetLevelRequired(bool adding) const
-	{
-		return adding ? ranktoset : ranktounset;
-	}
+	inline auto GetLevelRequired(bool adding) const { return adding ? ranktoset : ranktounset; }
 
 	/** Retrieves the syntax of the parameter for this mode. */
-	const std::string& GetSyntax() const { return syntax; }
+	const auto& GetSyntax() const { return syntax; }
 
 	friend class ModeParser;
 };
@@ -366,9 +351,10 @@ class CoreExport ModeHandler : public ServiceProvider
  * for example changing the topic on a channel having +t set requires a rank that is >= than the rank of a halfop,
  * but there is no such restriction when +t isn't set.
  */
-class CoreExport PrefixMode : public ModeHandler
+class CoreExport PrefixMode
+	: public ModeHandler
 {
- protected:
+protected:
 	/** The prefix character granted by this mode. '@' for op, '+' for voice, etc.
 	 * If 0, this mode does not have a visible prefix character.
 	 */
@@ -377,46 +363,36 @@ class CoreExport PrefixMode : public ModeHandler
 	/** The prefix rank of this mode, used to compare prefix
 	 * modes
 	 */
-	unsigned int prefixrank;
+	Rank prefixrank;
 
 	/** Whether a client with this prefix can remove it from themself. */
-	bool selfremove;
+	bool selfremove = true;
+
+public:
+	/** Sorts a container of PrefixMode* objects descending by their rank. */
+	struct Sorter final
+	{
+		bool operator()(const PrefixMode* lhs, const PrefixMode* rhs) const
+		{
+			return lhs->GetPrefixRank() > rhs->GetPrefixRank();
+		}
+	};
 
- public:
 	/**
 	 * Constructor
 	 * @param Creator The module creating this mode
 	 * @param Name The user-friendly one word name of the prefix mode, e.g.: "op", "voice"
 	 * @param ModeLetter The mode letter of this mode
-	 * @param Rank Rank given by this prefix mode, see explanation above
+	 * @param PrefixRank Rank given by this prefix mode, see explanation above
 	 * @param PrefixChar Prefix character, or 0 if the mode has no prefix character
 	 */
-	PrefixMode(Module* Creator, const std::string& Name, char ModeLetter, unsigned int Rank = 0, char PrefixChar = 0);
+	PrefixMode(Module* Creator, const std::string& Name, char ModeLetter, Rank PrefixRank = 0, char PrefixChar = 0);
 
-	/**
-	 * Called when a channel mode change access check for your mode occurs.
-	 * @param source Contains the user setting the mode.
-	 * @param channel contains the destination channel the modes are being set on.
-	 * @param parameter The parameter for your mode. This is modifiable.
-	 * @param adding This value is true when the mode is being set, or false when it is being unset.
-	 * @return allow, deny, or passthru to check against the required level
-	 */
-	ModResult AccessCheck(User* source, Channel* channel, std::string &parameter, bool adding) CXX11_OVERRIDE;
+	/** @copydoc ModeHandler::AccessCheck */
+	ModResult AccessCheck(User* source, Channel* channel, Modes::Change& change) override;
 
-	/**
-	 * Handles setting and unsetting the prefix mode.
-	 * Finds the given member of the given channel, if it's not found an error message is sent to 'source'
-	 * and MODEACTION_DENY is returned. Otherwise the mode change is attempted.
-	 * @param source Source of the mode change, an error message is sent to this user if the target is not found
-	 * @param dest Unused
-	 * @param channel The channel the mode change is happening on
-	 * @param param The nickname or uuid of the target user
-	 * @param adding True when the mode is being set, false when it is being unset
-	 * @return MODEACTION_ALLOW if the change happened, MODEACTION_DENY if no change happened
-	 * The latter occurs either when the member cannot be found or when the member already has this prefix set
-	 * (when setting) or doesn't have this prefix set (when unsetting).
-	 */
-	ModeAction OnModeChange(User* source, User* dest, Channel* channel, std::string& param, bool adding) CXX11_OVERRIDE;
+	/** @copydoc ModeHandler::OnModeChange */
+	bool OnModeChange(User* source, User* dest, Channel* channel, Modes::Change& change) override;
 
 	/**
 	 * Updates the configuration of this prefix.
@@ -425,15 +401,10 @@ class CoreExport PrefixMode : public ModeHandler
 	 * @param unsetrank The prefix rank required to set this unmode on channels.
 	 * @param selfrm Whether a client with this prefix can remove it from themself.
 	 */
-	void Update(unsigned int rank, unsigned int setrank, unsigned int unsetrank, bool selfrm);
-
-	/**
-	 * Removes this prefix mode from all users on the given channel
-	 * @param channel The channel which the server wants to remove your mode from
-	 * @param changelist Mode change list to populate with the removal of this mode
-	 */
-	void RemoveMode(Channel* channel, Modes::ChangeList& changelist) CXX11_OVERRIDE;
+	void Update(Rank rank, Rank setrank, Rank unsetrank, bool selfrm);
 
+	/** @copydoc ModeHandler::RemoveMode(Channel*,Modes::ChangeList&) */
+	void RemoveMode(Channel* channel, Modes::ChangeList& changelist) override;
 
 	/**
 	* Determines whether a user with this prefix mode can remove it.
@@ -454,7 +425,7 @@ class CoreExport PrefixMode : public ModeHandler
 	 * PrefixModeValue enum and Channel::GetPrefixValue() for
 	 * more information.
 	 */
-	unsigned int GetPrefixRank() const { return prefixrank; }
+	Rank GetPrefixRank() const { return prefixrank; }
 };
 
 /** A prebuilt mode handler which handles a simple user mode, e.g. no parameters, usable by any user, with no extra
@@ -462,16 +433,18 @@ class CoreExport PrefixMode : public ModeHandler
  * is already set and not allowing it to be unset if it is already unset.
  * An example of a simple user mode is user mode +w.
  */
-class CoreExport SimpleUserModeHandler : public ModeHandler
+class CoreExport SimpleUserMode
+	: public ModeHandler
 {
- public:
-	SimpleUserModeHandler(Module* Creator, const std::string& Name, char modeletter, bool operonly = false)
+public:
+	SimpleUserMode(Module* Creator, const std::string& Name, char modeletter, bool operonly = false)
 		: ModeHandler(Creator, Name, modeletter, PARAM_NONE, MODETYPE_USER)
 	{
 		oper = operonly;
 	}
 
-	ModeAction OnModeChange(User* source, User* dest, Channel* channel, std::string& parameter, bool adding) CXX11_OVERRIDE;
+	/** @copydoc ModeHandler::OnModeChange */
+	bool OnModeChange(User* source, User* dest, Channel* channel, Modes::Change& change) override;
 };
 
 /** A prebuilt mode handler which handles a simple channel mode, e.g. no parameters, usable by any user, with no extra
@@ -479,16 +452,19 @@ class CoreExport SimpleUserModeHandler : public ModeHandler
  * is already set and not allowing it to be unset if it is already unset.
  * An example of a simple channel mode is channel mode +s.
  */
-class CoreExport SimpleChannelModeHandler : public ModeHandler
+class CoreExport SimpleChannelMode
+	: public ModeHandler
 {
- public:
-	SimpleChannelModeHandler(Module* Creator, const std::string& Name, char modeletter, bool operonly = false)
+public:
+
+	SimpleChannelMode(Module* Creator, const std::string& Name, char modeletter, bool operonly = false)
 		: ModeHandler(Creator, Name, modeletter, PARAM_NONE, MODETYPE_CHANNEL)
 	{
 		oper = operonly;
 	}
 
-	ModeAction OnModeChange(User* source, User* dest, Channel* channel, std::string& parameter, bool adding) CXX11_OVERRIDE;
+	/** @copydoc ModeHandler::OnModeChange */
+	bool OnModeChange(User* source, User* dest, Channel* channel, Modes::Change& change) override;
 };
 
 /**
@@ -497,9 +473,10 @@ class CoreExport SimpleChannelModeHandler : public ModeHandler
  * and attach it to the mode using Server::AddModeWatcher and Server::DelModeWatcher.
  * A ModeWatcher will be called both before and after the mode change.
  */
-class CoreExport ModeWatcher : public classbase
+class CoreExport ModeWatcher
+	: public Cullable
 {
- private:
+private:
 	/**
 	 * The mode name this class is watching
 	 */
@@ -510,16 +487,18 @@ class CoreExport ModeWatcher : public classbase
 	 */
 	ModeType m_type;
 
- public:
+public:
 	ModuleRef creator;
+
 	/**
 	 * The constructor initializes the mode and the mode type
 	 */
 	ModeWatcher(Module* creator, const std::string& modename, ModeType type);
+
 	/**
 	 * The default destructor does nothing.
 	 */
-	virtual ~ModeWatcher();
+	~ModeWatcher() override;
 
 	/**
 	 * Get the mode name being watched
@@ -538,24 +517,22 @@ class CoreExport ModeWatcher : public classbase
 	 * @param source The sender of the mode
 	 * @param dest The target user for the mode, if you are watching a user mode
 	 * @param channel The target channel for the mode, if you are watching a channel mode
-	 * @param parameter The parameter of the mode, if the mode is supposed to have a parameter.
+	 * @param change Information regarding the mode change.
 	 * If you alter the parameter you are given, the mode handler will see your altered version
 	 * when it handles the mode.
-	 * @param adding True if the mode is being added and false if it is being removed
 	 * @return True to allow the mode change to go ahead, false to abort it. If you abort the
 	 * change, the mode handler (and ModeWatcher::AfterMode()) will never see the mode change.
 	 */
-	virtual bool BeforeMode(User* source, User* dest, Channel* channel, std::string& parameter, bool adding);
+	virtual bool BeforeMode(User* source, User* dest, Channel* channel, Modes::Change& change);
+
 	/**
 	 * After the mode character has been processed by the ModeHandler, this method will be called.
 	 * @param source The sender of the mode
 	 * @param dest The target user for the mode, if you are watching a user mode
 	 * @param channel The target channel for the mode, if you are watching a channel mode
-	 * @param parameter The parameter of the mode, if the mode is supposed to have a parameter.
-	 * You cannot alter the parameter here, as the mode handler has already processed it.
-	 * @param adding True if the mode is being added and false if it is being removed
+	 * @param change Information regarding the mode change.
 	 */
-	virtual void AfterMode(User* source, User* dest, Channel* channel, const std::string& parameter, bool adding);
+	virtual void AfterMode(User* source, User* dest, Channel* channel, const Modes::Change& change);
 };
 
 /** The mode parser handles routing of modes and handling of mode strings.
@@ -563,34 +540,34 @@ class CoreExport ModeWatcher : public classbase
  * parses client to server MODE strings for user and channel modes, and performs
  * processing for the 004 mode list numeric, amongst other things.
  */
-class CoreExport ModeParser : public fakederef<ModeParser>
+class CoreExport ModeParser final
 {
- public:
+public:
 	/** The maximum number of modes which can be created. */
-	static const ModeHandler::Id MODEID_MAX = 64;
+	static constexpr ModeHandler::Id MODEID_MAX = 64;
 
 	/** The maximum length of a mode parameter. */
-	static const size_t MODE_PARAM_MAX = 250;
+	static constexpr size_t MODE_PARAM_MAX = 250;
 
 	/** Type of the container that maps mode names to ModeHandlers
 	 */
-	typedef TR1NS::unordered_map<std::string, ModeHandler*, irc::insensitive, irc::StrHashComp> ModeHandlerMap;
+	typedef std::unordered_map<std::string, ModeHandler*, irc::insensitive, irc::StrHashComp> ModeHandlerMap;
 
- private:
+	/** Type of the container that contains whether a mode is set. */
+	typedef std::bitset<MODEID_MAX> ModeStatus;
+
+private:
 	/** Type of the container that maps mode names to ModeWatchers
 	 */
 	typedef insp::flat_multimap<std::string, ModeWatcher*> ModeWatcherMap;
 
 	/** Last item in the ModeType enum
 	 */
-	static const unsigned int MODETYPE_LAST = 2;
+	static constexpr unsigned int MODETYPE_LAST = 2;
 
-	/** Mode handlers for each mode, to access a handler subtract
-	 * 65 from the ascii value of the mode letter.
-	 * The upper bit of the value indicates if its a usermode
-	 * or a channel mode, so we have 256 of them not 64.
+	/** Mode handlers for each mode indexed by the mode index
 	 */
-	ModeHandler* modehandlers[MODETYPE_LAST][128];
+	ModeHandler* modehandlers[MODETYPE_LAST][MODEID_MAX];
 
 	/** An array of mode handlers indexed by the mode id
 	 */
@@ -628,15 +605,15 @@ class CoreExport ModeParser : public fakederef<ModeParser>
 	 * @param mcitem The actual mode change to attempt.
 	 * @param skipacl Whether to skip access checks for the mode change.
 	 */
-	ModeAction TryMode(User* user, User* usertarget, Channel* chantarget, Modes::Change& mcitem, bool skipacl);
+	bool TryMode(User* user, User* usertarget, Channel* chantarget, Modes::Change& mcitem, bool skipacl);
 
 	/** Allocates an unused id for the given mode type, throws a ModuleException if out of ids.
-	 * @param mt The type of the mode to allocate the id for
+	 * @param mh The mode to allocate the id for
 	 * @return The id
 	 */
-	ModeHandler::Id AllocateModeId(ModeType mt);
+	ModeHandler::Id AllocateModeId(ModeHandler* mh);
 
- public:
+public:
 	typedef std::vector<ListModeBase*> ListModeList;
 	typedef std::vector<PrefixMode*> PrefixModeList;
 
@@ -672,24 +649,27 @@ class CoreExport ModeParser : public fakederef<ModeParser>
 	};
 
 	ModeParser();
-	~ModeParser();
 
+	/** Determines whether the specified character is a valid mode. */
 	static bool IsModeChar(char chr);
 
+	/** Retrieves the index of the specified mode char within a mode bitset. */
+	static size_t GetModeIndex(char chr);
+
 	/** Tidy a banmask. This makes a banmask 'acceptable' if fields are left out.
 	 * E.g.
 	 *
 	 * nick -> nick!*@*
 	 *
-	 * nick!ident -> nick!ident@*
+	 * nick!user -> nick!user@*
 	 *
 	 * host.name -> *!*\@host.name
 	 *
-	 * ident\@host.name -> *!ident\@host.name
+	 * user\@host.name -> *!user\@host.name
 	 *
 	 * This method can be used on both IPV4 and IPV6 user masks.
 	 */
-	static void CleanMask(std::string &mask);
+	static void CleanMask(std::string& mask);
 
 	/** Gets the last mode change to be processed. */
 	const Modes::ChangeList& GetLastChangeList() const { return LastChangeList; }
@@ -750,7 +730,7 @@ class CoreExport ModeParser : public fakederef<ModeParser>
 	 * the element with this index are ignored.
 	 * @return Number of mode changes processed from changelist.
 	 */
-	unsigned int ProcessSingle(User* user, Channel* targetchannel, User* targetuser, Modes::ChangeList& changelist, ModeProcessFlag flags = MODE_NONE, unsigned int beginindex = 0);
+	size_t ProcessSingle(User* user, Channel* targetchannel, User* targetuser, Modes::ChangeList& changelist, ModeProcessFlag flags = MODE_NONE, size_t beginindex = 0);
 
 	/** Turn a list of parameters compatible with the format of the MODE command into
 	 * Modes::ChangeList form. All modes are processed, regardless of max modes. Unknown modes
@@ -767,7 +747,7 @@ class CoreExport ModeParser : public fakederef<ModeParser>
 	 * @param endindex Index of the first element that is not part of the MODE list. By default,
 	 * the entire container is considered part of the MODE list.
 	 */
-	void ModeParamsToChangeList(User* user, ModeType type, const std::vector<std::string>& parameters, Modes::ChangeList& changelist, unsigned int beginindex = 1, unsigned int endindex = UINT_MAX);
+	void ModeParamsToChangeList(User* user, ModeType type, const std::vector<std::string>& parameters, Modes::ChangeList& changelist, size_t beginindex = 1, size_t endindex = SIZE_MAX);
 
 	/** Find the mode handler for a given mode name and type.
 	 * @param modename The mode name to search for.
@@ -781,7 +761,7 @@ class CoreExport ModeParser : public fakederef<ModeParser>
 	 * @param mt type of mode to search for, user or channel
 	 * @returns a pointer to a ModeHandler class, or NULL of there isn't a handler for the given mode
 	 */
-	ModeHandler* FindMode(unsigned const char modeletter, ModeType mt);
+	ModeHandler* FindMode(unsigned char modeletter, ModeType mt);
 
 	/** Find the mode handler for the given prefix mode
 	 * @param modeletter The mode letter to search for
@@ -793,27 +773,14 @@ class CoreExport ModeParser : public fakederef<ModeParser>
 	 * @param rank The rank to search for prefix modes near.
 	 * @return A pointer to the PrefixMode or NULL if a prefix mode wasn't found.
 	 */
-	PrefixMode* FindNearestPrefixMode(unsigned int rank);
+	PrefixMode* FindNearestPrefixMode(ModeHandler::Rank rank) const;
 
 	/** Find a mode handler by its prefix.
 	 * If there is no mode handler with the given prefix, NULL will be returned.
 	 * @param pfxletter The prefix to find, e.g. '@'
 	 * @return The mode handler which handles this prefix, or NULL if there is none.
 	 */
-	PrefixMode* FindPrefix(unsigned const char pfxletter);
-
-	/** Generates a list of modes, comma separated by type:
-	 *  1; Listmodes EXCEPT those with a prefix
-	 *  2; Modes that take a param when adding or removing
-	 *  3; Modes that only take a param when adding
-	 *  4; Modes that dont take a param
-	 */
-	std::string GiveModeList(ModeType mt);
-
-	/** This returns the PREFIX=(ohv)@%+ section of the 005 numeric, or
-	 * just the "@%+" part if the parameter false
-	 */
-	std::string BuildPrefixes(bool lettersAndModes = true);
+	PrefixMode* FindPrefix(unsigned char pfxletter) const;
 
 	/** Get a list of all mode handlers that inherit from ListModeBase
 	 * @return A list containing ListModeBase modes
@@ -841,30 +808,30 @@ class CoreExport ModeParser : public fakederef<ModeParser>
 
 inline PrefixMode* ModeHandler::IsPrefixMode()
 {
-	return (this->type_id == MC_PREFIX ? static_cast<PrefixMode*>(this) : NULL);
+	return (this->type_id == MC_PREFIX ? static_cast<PrefixMode*>(this) : nullptr);
 }
 
 inline const PrefixMode* ModeHandler::IsPrefixMode() const
 {
-	return (this->type_id == MC_PREFIX ? static_cast<const PrefixMode*>(this) : NULL);
+	return (this->type_id == MC_PREFIX ? static_cast<const PrefixMode*>(this) : nullptr);
 }
 
 inline ListModeBase* ModeHandler::IsListModeBase()
 {
-	return (this->type_id == MC_LIST ? reinterpret_cast<ListModeBase*>(this) : NULL);
+	return (this->type_id == MC_LIST ? reinterpret_cast<ListModeBase*>(this) : nullptr);
 }
 
 inline const ListModeBase* ModeHandler::IsListModeBase() const
 {
-	return (this->type_id == MC_LIST ? reinterpret_cast<const ListModeBase*>(this) : NULL);
+	return (this->type_id == MC_LIST ? reinterpret_cast<const ListModeBase*>(this) : nullptr);
 }
 
 inline ParamModeBase* ModeHandler::IsParameterMode()
 {
-	return (this->type_id == MC_PARAM ? reinterpret_cast<ParamModeBase*>(this) : NULL);
+	return (this->type_id == MC_PARAM ? reinterpret_cast<ParamModeBase*>(this) : nullptr);
 }
 
 inline const ParamModeBase* ModeHandler::IsParameterMode() const
 {
-	return (this->type_id == MC_PARAM ? reinterpret_cast<const ParamModeBase*>(this) : NULL);
+	return (this->type_id == MC_PARAM ? reinterpret_cast<const ParamModeBase*>(this) : nullptr);
 }
