@@ -1,7 +1,7 @@
 #
 # InspIRCd -- Internet Relay Chat Daemon
 #
-#   Copyright (C) 2016-2024 Sadie Powell <sadie@witchery.services>
+#   Copyright (C) 2016-2025 Sadie Powell <sadie@witchery.services>
 #
 # This file is part of InspIRCd.  InspIRCd is free software: you can
 # redistribute it and/or modify it under the terms of the GNU General Public
@@ -345,6 +345,24 @@ sub __function_require_library {
 	return $ok;
 }
 
+sub __function_require_environment {
+	my ($file, $name, $value) = @_;
+
+	# Check for an inverted match.
+	my ($ok, $err) = ("", undef);
+	if ($name =~ s/^!//) {
+		($ok, $err) = ($err, $ok);
+	}
+
+	return $err unless defined $ENV{$name};
+	if (defined $value) {
+		return $err unless $ENV{$name} eq $value;
+	}
+
+	# Requirement directives don't change anything directly.
+	return $ok;
+}
+
 sub __function_warning {
 	my ($file, @messages) = @_;
 	print_warning @messages;
