@@ -1,9 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2015, 2017-2023 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2013-2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2015, 2018-2023, 2025 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2014-2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2013, 2015-2016, 2021 Adam <Adam@anope.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -19,9 +20,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+
 #include "inspircd.h"
 #include "modules/dns.h"
-#include <iostream>
+#include "modules/stats.h"
+#include "stringutils.h"
+
 #include <fstream>
 
 #ifdef _WIN32
@@ -36,18 +40,20 @@ namespace DNS
 	const unsigned int MAX_REQUEST_ID = 0xFFFF;
 }
 
-using namespace DNS;
-
 /** A full packet sent or received to/from the nameserver
  */
-class Packet : public Query
+class Packet final
+	: public DNS::Query
 {
+private:
+	const Module* creator;
+
 	void PackName(unsigned char* output, unsigned short output_size, unsigned short& pos, const std::string& name)
 	{
 		if (pos + name.length() + 2 > output_size)
-			throw Exception("Unable to pack name");
+			throw DNS::Exception(creator, "Unable to pack name");
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Packing name " + name);
+		ServerInstance->Logs.Debug(MODNAME, "Packing name {}", name);
 
 		irc::sepstream sep(name, '.');
 		std::string token;
@@ -65,11 +71,12 @@ class Packet : public Query
 	std::string UnpackName(const unsigned char* input, unsigned short input_size, unsigned short& pos)
 	{
 		std::string name;
-		unsigned short pos_ptr = pos, lowest_ptr = input_size;
+		unsigned short pos_ptr = pos;
+		unsigned short lowest_ptr = input_size;
 		bool compressed = false;
 
 		if (pos_ptr >= input_size)
-			throw Exception("Unable to unpack name - no input");
+			throw DNS::Exception(creator, "Unable to unpack name - no input");
 
 		while (input[pos_ptr] > 0)
 		{
@@ -78,12 +85,12 @@ class Packet : public Query
 			if (offset & POINTER)
 			{
 				if ((offset & POINTER) != POINTER)
-					throw Exception("Unable to unpack name - bogus compression header");
+					throw DNS::Exception(creator, "Unable to unpack name - bogus compression header");
 				if (pos_ptr + 1 >= input_size)
-					throw Exception("Unable to unpack name - bogus compression header");
+					throw DNS::Exception(creator, "Unable to unpack name - bogus compression header");
 
 				/* Place pos at the second byte of the first (farthest) compression pointer */
-				if (compressed == false)
+				if (!compressed)
 				{
 					++pos;
 					compressed = true;
@@ -93,20 +100,20 @@ class Packet : public Query
 
 				/* Pointers can only go back */
 				if (pos_ptr >= lowest_ptr)
-					throw Exception("Unable to unpack name - bogus compression pointer");
+					throw DNS::Exception(creator, "Unable to unpack name - bogus compression pointer");
 				lowest_ptr = pos_ptr;
 			}
 			else
 			{
 				if (pos_ptr + offset + 1 >= input_size)
-					throw Exception("Unable to unpack name - offset too large");
+					throw DNS::Exception(creator, "Unable to unpack name - offset too large");
 				if (!name.empty())
 					name += ".";
 				for (unsigned i = 1; i <= offset; ++i)
 					name += input[pos_ptr + i];
 
 				pos_ptr += offset + 1;
-				if (compressed == false)
+				if (!compressed)
 					/* Move up pos */
 					pos = pos_ptr;
 			}
@@ -116,23 +123,23 @@ class Packet : public Query
 		++pos;
 
 		if (name.empty())
-			throw Exception("Unable to unpack name - no name");
+			throw DNS::Exception(creator, "Unable to unpack name - no name");
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Unpack name " + name);
+		ServerInstance->Logs.Debug(MODNAME, "Unpack name {}", name);
 
 		return name;
 	}
 
-	Question UnpackQuestion(const unsigned char* input, unsigned short input_size, unsigned short& pos)
+	DNS::Question UnpackQuestion(const unsigned char* input, unsigned short input_size, unsigned short& pos)
 	{
-		Question q;
+		DNS::Question q;
 
 		q.name = this->UnpackName(input, input_size, pos);
 
 		if (pos + 4 > input_size)
-			throw Exception("Unable to unpack question");
+			throw DNS::Exception(creator, "Unable to unpack question");
 
-		q.type = static_cast<QueryType>(input[pos] << 8 | input[pos + 1]);
+		q.type = static_cast<DNS::QueryType>(input[pos] << 8 | input[pos + 1]);
 		pos += 2;
 
 		// Skip over query class code
@@ -141,12 +148,12 @@ class Packet : public Query
 		return q;
 	}
 
-	ResourceRecord UnpackResourceRecord(const unsigned char* input, unsigned short input_size, unsigned short& pos)
+	DNS::ResourceRecord UnpackResourceRecord(const unsigned char* input, unsigned short input_size, unsigned short& pos)
 	{
-		ResourceRecord record = static_cast<ResourceRecord>(this->UnpackQuestion(input, input_size, pos));
+		auto record = static_cast<DNS::ResourceRecord>(this->UnpackQuestion(input, input_size, pos));
 
 		if (pos + 6 > input_size)
-			throw Exception("Unable to unpack resource record");
+			throw DNS::Exception(creator, "Unable to unpack resource record");
 
 		record.ttl = (input[pos] << 24) | (input[pos + 1] << 16) | (input[pos + 2] << 8) | input[pos + 3];
 		pos += 4;
@@ -156,14 +163,12 @@ class Packet : public Query
 
 		switch (record.type)
 		{
-			case QUERY_A:
+			case DNS::QUERY_A:
 			{
 				if (pos + 4 > input_size)
-					throw Exception("Unable to unpack resource record");
+					throw DNS::Exception(creator, "Unable to unpack A resource record");
 
 				irc::sockets::sockaddrs addrs;
-				memset(&addrs, 0, sizeof(addrs));
-
 				addrs.in4.sin_family = AF_INET;
 				addrs.in4.sin_addr.s_addr = input[pos] | (input[pos + 1] << 8) | (input[pos + 2] << 16)  | (input[pos + 3] << 24);
 				pos += 4;
@@ -171,14 +176,12 @@ class Packet : public Query
 				record.rdata = addrs.addr();
 				break;
 			}
-			case QUERY_AAAA:
+			case DNS::QUERY_AAAA:
 			{
 				if (pos + 16 > input_size)
-					throw Exception("Unable to unpack resource record");
+					throw DNS::Exception(creator, "Unable to unpack AAAA resource record");
 
 				irc::sockets::sockaddrs addrs;
-				memset(&addrs, 0, sizeof(addrs));
-
 				addrs.in6.sin6_family = AF_INET6;
 				for (int j = 0; j < 16; ++j)
 					addrs.in6.sin6_addr.s6_addr[j] = input[pos + j];
@@ -188,32 +191,56 @@ class Packet : public Query
 
 				break;
 			}
-			case QUERY_CNAME:
-			case QUERY_PTR:
+			case DNS::QUERY_CNAME:
+			case DNS::QUERY_PTR:
 			{
 				record.rdata = this->UnpackName(input, input_size, pos);
-				if (!InspIRCd::IsHost2(record.rdata, true))
-					throw Exception("Invalid name"); // XXX: Causes the request to time out
+				if (!InspIRCd::IsHost(record.rdata, true))
+					throw DNS::Exception(creator, "Invalid name in CNAME/PTR resource record");
 
 				break;
 			}
-			case QUERY_TXT:
+			case DNS::QUERY_TXT:
 			{
 				if (pos + rdlength > input_size)
-					throw Exception("Unable to unpack txt resource record");
+					throw DNS::Exception(creator, "Unable to unpack TXT resource record");
 
-				record.rdata = std::string(reinterpret_cast<const char *>(input + pos), rdlength);
+				record.rdata = std::string(reinterpret_cast<const char* >(input + pos), rdlength);
 				pos += rdlength;
 
 				if (record.rdata.find_first_of("\r\n\0", 0, 3) != std::string::npos)
-					throw Exception("Invalid character in txt record");
+					throw DNS::Exception(creator, "Invalid character in TXT resource record");
 
 				break;
 			}
+			case DNS::QUERY_SRV:
+			{
+				if (rdlength < 6 || pos + rdlength > input_size)
+					throw DNS::Exception(creator, "Unable to unpack SRV resource record");
+
+				auto srv = std::make_shared<DNS::Record::SRV>();
+
+				srv->priority = input[pos] << 8 | input[pos + 1];
+				pos += 2;
+
+				srv->weight = input[pos] << 8 | input[pos + 1];
+				pos += 2;
+
+				srv->port = input[pos] << 8 | input[pos + 1];
+				pos += 2;
+
+				srv->host = this->UnpackName(input, input_size, pos);
+				if (!InspIRCd::IsHost(srv->host, true))
+					throw DNS::Exception(creator, "Invalid name in SRV resource record");
+
+				record.rdata = INSP_FORMAT("{} {} {} {}", srv->priority, srv->weight, srv->port, srv->host);
+				record.rdataobj = srv;
+				break;
+			}
 			default:
 			{
 				if (pos + rdlength > input_size)
-					throw Exception("Unable to skip resource record");
+					throw DNS::Exception(creator, "Unable to skip resource record");
 
 				pos += rdlength;
 				break;
@@ -221,29 +248,31 @@ class Packet : public Query
 		}
 
 		if (!record.name.empty() && !record.rdata.empty())
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, record.name + " -> " + record.rdata);
+			ServerInstance->Logs.Debug(MODNAME, "{} -> {}", record.name, record.rdata);
 
 		return record;
 	}
 
- public:
-	static const int POINTER = 0xC0;
-	static const int LABEL = 0x3F;
-	static const int HEADER_LENGTH = 12;
+public:
+	static constexpr int POINTER = 0xC0;
+	static constexpr int LABEL = 0x3F;
+	static constexpr int HEADER_LENGTH = 12;
 
 	/* ID for this packet */
-	RequestId id;
-	/* Flags on the packet */
-	unsigned short flags;
+	DNS::RequestId id = 0;
 
-	Packet() : id(0), flags(0)
+	/* Flags on the packet */
+	unsigned short flags = 0;
+
+	Packet(const Module* mod)
+		: creator(mod)
 	{
 	}
 
-	void Fill(const unsigned char* input, const unsigned short len)
+	void Fill(const unsigned char* input, unsigned short len)
 	{
 		if (len < HEADER_LENGTH)
-			throw Exception("Unable to fill packet");
+			throw DNS::Exception(creator, "Unable to fill packet");
 
 		unsigned short packet_pos = 0;
 
@@ -265,10 +294,11 @@ class Packet : public Query
 		unsigned short arcount = (input[packet_pos] << 8) | input[packet_pos + 1];
 		packet_pos += 2;
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "qdcount: " + ConvToStr(qdcount) + " ancount: " + ConvToStr(ancount) + " nscount: " + ConvToStr(nscount) + " arcount: " + ConvToStr(arcount));
+		ServerInstance->Logs.Debug(MODNAME, "qdcount: {} ancount: {} nscount: {} arcount: {}",
+			qdcount, ancount, nscount, arcount);
 
 		if (qdcount != 1)
-			throw Exception("Question count != 1 in incoming packet");
+			throw DNS::Exception(creator, "Question count != 1 in incoming packet");
 
 		this->question = this->UnpackQuestion(input, len, packet_pos);
 
@@ -279,7 +309,7 @@ class Packet : public Query
 	unsigned short Pack(unsigned char* output, unsigned short output_size)
 	{
 		if (output_size < HEADER_LENGTH)
-			throw Exception("Unable to pack oversized packet header");
+			throw DNS::Exception(creator, "Unable to pack oversized packet header");
 
 		unsigned short pos = 0;
 
@@ -297,25 +327,24 @@ class Packet : public Query
 		output[pos++] = 0;
 
 		{
-			Question& q = this->question;
+			auto& q = this->question;
 
-			if (q.type == QUERY_PTR)
+			if (q.type == DNS::QUERY_PTR)
 			{
-				irc::sockets::sockaddrs ip;
-				if (!irc::sockets::aptosa(q.name, 0, ip))
-					throw Exception("Unable to pack packet with malformed IP for PTR lookup");
+				irc::sockets::sockaddrs ip(false);
+				if (!ip.from_ip(q.name))
+					throw DNS::Exception(creator, "Unable to pack packet with malformed IP for PTR lookup");
 
 				if (q.name.find(':') != std::string::npos)
 				{
 					// ::1 => 1.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.ip6.arpa
-					static const char* const hex = "0123456789abcdef";
 					char reverse_ip[128];
 					unsigned reverse_ip_count = 0;
 					for (int j = 15; j >= 0; --j)
 					{
-						reverse_ip[reverse_ip_count++] = hex[ip.in6.sin6_addr.s6_addr[j] & 0xF];
+						reverse_ip[reverse_ip_count++] = Hex::TABLE_LOWER[ip.in6.sin6_addr.s6_addr[j] & 0xF];
 						reverse_ip[reverse_ip_count++] = '.';
-						reverse_ip[reverse_ip_count++] = hex[ip.in6.sin6_addr.s6_addr[j] >> 4];
+						reverse_ip[reverse_ip_count++] = Hex::TABLE_LOWER[ip.in6.sin6_addr.s6_addr[j] >> 4];
 						reverse_ip[reverse_ip_count++] = '.';
 					}
 					reverse_ip[reverse_ip_count++] = 0;
@@ -326,7 +355,7 @@ class Packet : public Query
 				else
 				{
 					// 127.0.0.1 => 1.0.0.127.in-addr.arpa
-					unsigned long forward = ip.in4.sin_addr.s_addr;
+					unsigned int forward = ip.in4.sin_addr.s_addr;
 					ip.in4.sin_addr.s_addr = forward << 24 | (forward & 0xFF00) << 8 | (forward & 0xFF0000) >> 8 | forward >> 24;
 
 					q.name = ip.addr() + ".in-addr.arpa";
@@ -336,7 +365,7 @@ class Packet : public Query
 			this->PackName(output, output_size, pos, q.name);
 
 			if (pos + 4 >= output_size)
-				throw Exception("Unable to pack oversized packet body");
+				throw DNS::Exception(creator, "Unable to pack oversized packet body");
 
 			short s = htons(q.type);
 			memcpy(&output[pos], &s, 2);
@@ -351,22 +380,25 @@ class Packet : public Query
 	}
 };
 
-class MyManager : public Manager, public Timer, public EventHandler
+class MyManager final
+	: public DNS::Manager
+	, public Timer
+	, public EventHandler
 {
-	typedef TR1NS::unordered_map<Question, Query, Question::hash> cache_map;
+	typedef std::unordered_map<DNS::Question, DNS::Query, DNS::Question::hash> cache_map;
 	cache_map cache;
 
 	irc::sockets::sockaddrs myserver;
-	bool unloading;
+	bool unloading = false;
 
 	/** Maximum number of entries in cache
 	 */
-	static const unsigned int MAX_CACHE_SIZE = 1000;
+	static constexpr unsigned int MAX_CACHE_SIZE = 1000;
 
-	static bool IsExpired(const Query& record, time_t now = ServerInstance->Time())
+	static bool IsExpired(const DNS::Query& record)
 	{
-		const ResourceRecord& req = record.answers[0];
-		return (req.created + static_cast<time_t>(req.ttl) < now);
+		const auto& req = record.answers[0];
+		return (req.created + static_cast<time_t>(req.ttl) < ServerInstance->Time());
 	}
 
 	/** Check the DNS cache to see if request can be handled by a cached result
@@ -374,20 +406,20 @@ class MyManager : public Manager, public Timer, public EventHandler
 	 */
 	bool CheckCache(DNS::Request* req, const DNS::Question& question)
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_SPARSE, "cache: Checking cache for " + question.name);
+		ServerInstance->Logs.Critical(MODNAME, "cache: Checking cache for {}", question.name);
 
 		cache_map::iterator it = this->cache.find(question);
 		if (it == this->cache.end())
 			return false;
 
-		Query& record = it->second;
+		auto& record = it->second;
 		if (IsExpired(record))
 		{
 			this->cache.erase(it);
 			return false;
 		}
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "cache: Using cached result for " + question.name);
+		ServerInstance->Logs.Debug(MODNAME, "cache: Using cached result for {}", question.name);
 		record.cached = true;
 		req->OnLookupComplete(&record);
 		return true;
@@ -396,53 +428,57 @@ class MyManager : public Manager, public Timer, public EventHandler
 	/** Add a record to the dns cache
 	 * @param r The record
 	 */
-	void AddCache(Query& r)
+	void AddCache(DNS::Query& r)
 	{
 		if (cache.size() >= MAX_CACHE_SIZE)
 			cache.clear();
 
 		// Determine the lowest TTL value and use that as the TTL of the cache entry
 		unsigned int cachettl = UINT_MAX;
-		for (std::vector<ResourceRecord>::const_iterator i = r.answers.begin(); i != r.answers.end(); ++i)
+		for (const auto& rr : r.answers)
 		{
-			const ResourceRecord& rr = *i;
 			if (rr.ttl < cachettl)
 				cachettl = rr.ttl;
 		}
 
-		cachettl = std::min(cachettl, (unsigned int)5*60);
-		ResourceRecord& rr = r.answers.front();
+		cachettl = std::min<unsigned int>(cachettl, 5*60);
+		auto& rr = r.answers.front();
 		// Set TTL to what we've determined to be the lowest
 		rr.ttl = cachettl;
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "cache: added cache for " + rr.name + " -> " + rr.rdata + " ttl: " + ConvToStr(rr.ttl));
+		ServerInstance->Logs.Debug(MODNAME, "cache: added cache for {} -> {} ttl: {}", rr.name, rr.rdata, rr.ttl);
 		this->cache[r.question] = r;
 	}
 
- public:
-	DNS::Request* requests[MAX_REQUEST_ID+1];
+public:
+	DNS::Request* requests[DNS::MAX_REQUEST_ID + 1];
+	size_t stats_total = 0;
+	size_t stats_success = 0;
+	size_t stats_failure = 0;
+	unsigned long timeout = 0;
 
-	MyManager(Module* c) : Manager(c), Timer(5*60, true)
-		, unloading(false)
+	MyManager(Module* c)
+		: Manager(c)
+		, Timer(5*60, true)
 	{
-		for (unsigned int i = 0; i <= MAX_REQUEST_ID; ++i)
-			requests[i] = NULL;
+		for (unsigned int i = 0; i <= DNS::MAX_REQUEST_ID; ++i)
+			requests[i] = nullptr;
 		ServerInstance->Timers.AddTimer(this);
 	}
 
-	~MyManager()
+	~MyManager() override
 	{
 		// Ensure Process() will fail for new requests
 		Close();
 		unloading = true;
 
-		for (unsigned int i = 0; i <= MAX_REQUEST_ID; ++i)
+		for (unsigned int i = 0; i <= DNS::MAX_REQUEST_ID; ++i)
 		{
 			DNS::Request* request = requests[i];
 			if (!request)
 				continue;
 
-			Query rr(request->question);
-			rr.error = ERROR_UNKNOWN;
+			DNS::Query rr(request->question);
+			rr.error = DNS::ERROR_UNKNOWN;
 			request->OnError(&rr);
 
 			delete request;
@@ -462,24 +498,30 @@ class MyManager : public Manager, public Timer, public EventHandler
 		cache.clear();
 	}
 
-	void Process(DNS::Request* req) CXX11_OVERRIDE
+	unsigned long GetDefaultTimeout() const override
+	{
+		return timeout;
+	}
+
+	void Process(DNS::Request* req) override
 	{
 		if ((unloading) || (req->creator->dying))
-			throw Exception("Module is being unloaded");
+			throw DNS::Exception(creator, "Module is being unloaded");
 
 		if (!HasFd())
 		{
-			Query rr(req->question);
-			rr.error = ERROR_DISABLED;
+			DNS::Query rr(req->question);
+			rr.error = DNS::ERROR_DISABLED;
 			req->OnError(&rr);
 			return;
 		}
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Processing request to lookup " + req->question.name + " of type " + ConvToStr(req->question.type) + " to " + this->myserver.addr());
+		ServerInstance->Logs.Debug(MODNAME, "Processing request to lookup {} of type {} to {}",
+			req->question.name, (int)req->question.type, this->myserver.addr());
 
 		/* Create an id */
 		unsigned int tries = 0;
-		int id;
+		long id;
 		do
 		{
 			id = ServerInstance->GenRandomInt(DNS::MAX_REQUEST_ID+1);
@@ -499,7 +541,7 @@ class MyManager : public Manager, public Timer, public EventHandler
 				}
 
 				if (id == -1)
-					throw Exception("DNS: All ids are in use");
+					throw DNS::Exception(creator, "DNS: All ids are in use");
 
 				break;
 			}
@@ -509,8 +551,8 @@ class MyManager : public Manager, public Timer, public EventHandler
 		req->id = id;
 		this->requests[req->id] = req;
 
-		Packet p;
-		p.flags = QUERYFLAGS_RD;
+		Packet p(creator);
+		p.flags = DNS::QUERYFLAGS_RD;
 		p.id = req->id;
 		p.question = req->question;
 
@@ -522,7 +564,7 @@ class MyManager : public Manager, public Timer, public EventHandler
 		 */
 		if (req->use_cache && this->CheckCache(req, p.question))
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Using cached result");
+			ServerInstance->Logs.Debug(MODNAME, "Using cached result");
 			delete req;
 			return;
 		}
@@ -532,79 +574,81 @@ class MyManager : public Manager, public Timer, public EventHandler
 		req->question.name = p.question.name;
 
 		if (SocketEngine::SendTo(this, buffer, len, 0, this->myserver) != len)
-			throw Exception("DNS: Unable to send query");
+			throw DNS::Exception(creator, "DNS: Unable to send query");
 
 		// Add timer for timeout
 		ServerInstance->Timers.AddTimer(req);
 	}
 
-	void RemoveRequest(DNS::Request* req) CXX11_OVERRIDE
+	void RemoveRequest(DNS::Request* req) override
 	{
 		if (requests[req->id] == req)
-			requests[req->id] = NULL;
+			requests[req->id] = nullptr;
 	}
 
-	std::string GetErrorStr(Error e) CXX11_OVERRIDE
+	std::string GetErrorStr(DNS::Error e) override
 	{
 		switch (e)
 		{
-			case ERROR_UNLOADED:
+			case DNS::ERROR_UNLOADED:
 				return "Module is unloading";
-			case ERROR_TIMEDOUT:
+			case DNS::ERROR_TIMEDOUT:
 				return "Request timed out";
-			case ERROR_NOT_AN_ANSWER:
-			case ERROR_NONSTANDARD_QUERY:
-			case ERROR_FORMAT_ERROR:
-			case ERROR_MALFORMED:
+			case DNS::ERROR_NOT_AN_ANSWER:
+			case DNS::ERROR_NONSTANDARD_QUERY:
+			case DNS::ERROR_FORMAT_ERROR:
+			case DNS::ERROR_MALFORMED:
 				return "Malformed answer";
-			case ERROR_SERVER_FAILURE:
-			case ERROR_NOT_IMPLEMENTED:
-			case ERROR_REFUSED:
-			case ERROR_INVALIDTYPE:
+			case DNS::ERROR_SERVER_FAILURE:
+			case DNS::ERROR_NOT_IMPLEMENTED:
+			case DNS::ERROR_REFUSED:
+			case DNS::ERROR_INVALIDTYPE:
 				return "Nameserver failure";
-			case ERROR_DOMAIN_NOT_FOUND:
-			case ERROR_NO_RECORDS:
+			case DNS::ERROR_DOMAIN_NOT_FOUND:
+			case DNS::ERROR_NO_RECORDS:
 				return "Domain not found";
-			case ERROR_DISABLED:
+			case DNS::ERROR_DISABLED:
 				return "DNS lookups are disabled";
-			case ERROR_NONE:
-			case ERROR_UNKNOWN:
-			default:
-				return "Unknown error";
+			case DNS::ERROR_NONE:
+			case DNS::ERROR_UNKNOWN:
+				break;
 		}
+		return "Unknown error";
 	}
 
-	std::string GetTypeStr(QueryType qt) CXX11_OVERRIDE
+	std::string GetTypeStr(DNS::QueryType qt) override
 	{
 		switch (qt)
 		{
-			case QUERY_A:
+			case DNS::QUERY_A:
 				return "A";
-			case QUERY_AAAA:
+			case DNS::QUERY_AAAA:
 				return "AAAA";
-			case QUERY_CNAME:
+			case DNS::QUERY_CNAME:
 				return "CNAME";
-			case QUERY_PTR:
+			case DNS::QUERY_PTR:
 				return "PTR";
-			case QUERY_TXT:
+			case DNS::QUERY_TXT:
 				return "TXT";
+			case DNS::QUERY_SRV:
+				return "SRV";
 			default:
 				return "UNKNOWN";
 		}
 	}
 
-	void OnEventHandlerError(int errcode) CXX11_OVERRIDE
+	void OnEventHandlerError(int errcode) override
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "UDP socket got an error event");
+		ServerInstance->Logs.Debug(MODNAME, "UDP socket got an error event");
 	}
 
-	void OnEventHandlerRead() CXX11_OVERRIDE
+	void OnEventHandlerRead() override
 	{
 		unsigned char buffer[524];
-		irc::sockets::sockaddrs from;
+		irc::sockets::sockaddrs from(false);
 		socklen_t x = sizeof(from);
 
-		int length = SocketEngine::RecvFrom(this, buffer, sizeof(buffer), 0, &from.sa, &x);
+		ssize_t length = SocketEngine::RecvFrom(this, buffer, sizeof(buffer), 0, &from.sa, &x);
 
 		if (length < Packet::HEADER_LENGTH)
 			return;
@@ -613,12 +657,12 @@ class MyManager : public Manager, public Timer, public EventHandler
 		{
 			std::string server1 = from.str();
 			std::string server2 = myserver.str();
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Got a result from the wrong server! Bad NAT or DNS forging attempt? '%s' != '%s'",
-				server1.c_str(), server2.c_str());
+			ServerInstance->Logs.Debug(MODNAME, "Got a result from the wrong server! Bad NAT or DNS forging attempt? '{}' != '{}'",
+				server1, server2);
 			return;
 		}
 
-		Packet recv_packet;
+		Packet recv_packet(creator);
 		bool valid = false;
 
 		try
@@ -626,101 +670,101 @@ class MyManager : public Manager, public Timer, public EventHandler
 			recv_packet.Fill(buffer, length);
 			valid = true;
 		}
-		catch (Exception& ex)
+		catch (const DNS::Exception& ex)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, ex.GetReason());
+			ServerInstance->Logs.Debug(MODNAME, ex.GetReason());
 		}
 
 		// recv_packet.id must be filled in here
 		DNS::Request* request = this->requests[recv_packet.id];
-		if (request == NULL)
+		if (!request)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Received an answer for something we didn't request");
+			ServerInstance->Logs.Debug(MODNAME, "Received an answer for something we didn't request");
 			return;
 		}
 
 		if (request->question != recv_packet.question)
 		{
 			// This can happen under high latency, drop it silently, do not fail the request
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Received an answer that isn't for a question we asked");
+			ServerInstance->Logs.Debug(MODNAME, "Received an answer that isn't for a question we asked");
 			return;
 		}
 
 		if (!valid)
 		{
-			ServerInstance->stats.DnsBad++;
-			recv_packet.error = ERROR_MALFORMED;
+			this->stats_failure++;
+			recv_packet.error = DNS::ERROR_MALFORMED;
 			request->OnError(&recv_packet);
 		}
-		else if (recv_packet.flags & QUERYFLAGS_OPCODE)
+		else if (recv_packet.flags & DNS::QUERYFLAGS_OPCODE)
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Received a nonstandard query");
-			ServerInstance->stats.DnsBad++;
-			recv_packet.error = ERROR_NONSTANDARD_QUERY;
+			ServerInstance->Logs.Debug(MODNAME, "Received a nonstandard query");
+			this->stats_failure++;
+			recv_packet.error = DNS::ERROR_NONSTANDARD_QUERY;
 			request->OnError(&recv_packet);
 		}
-		else if (!(recv_packet.flags & QUERYFLAGS_QR) || (recv_packet.flags & QUERYFLAGS_RCODE))
+		else if (!(recv_packet.flags & DNS::QUERYFLAGS_QR) || (recv_packet.flags & DNS::QUERYFLAGS_RCODE))
 		{
-			Error error = ERROR_UNKNOWN;
+			auto error = DNS::ERROR_UNKNOWN;
 
-			switch (recv_packet.flags & QUERYFLAGS_RCODE)
+			switch (recv_packet.flags & DNS::QUERYFLAGS_RCODE)
 			{
 				case 1:
-					ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "format error");
-					error = ERROR_FORMAT_ERROR;
+					ServerInstance->Logs.Debug(MODNAME, "format error");
+					error = DNS::ERROR_FORMAT_ERROR;
 					break;
 				case 2:
-					ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "server error");
-					error = ERROR_SERVER_FAILURE;
+					ServerInstance->Logs.Debug(MODNAME, "server error");
+					error = DNS::ERROR_SERVER_FAILURE;
 					break;
 				case 3:
-					ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "domain not found");
-					error = ERROR_DOMAIN_NOT_FOUND;
+					ServerInstance->Logs.Debug(MODNAME, "domain not found");
+					error = DNS::ERROR_DOMAIN_NOT_FOUND;
 					break;
 				case 4:
-					ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "not implemented");
-					error = ERROR_NOT_IMPLEMENTED;
+					ServerInstance->Logs.Debug(MODNAME, "not implemented");
+					error = DNS::ERROR_NOT_IMPLEMENTED;
 					break;
 				case 5:
-					ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "refused");
-					error = ERROR_REFUSED;
+					ServerInstance->Logs.Debug(MODNAME, "refused");
+					error = DNS::ERROR_REFUSED;
 					break;
 				default:
 					break;
 			}
 
-			ServerInstance->stats.DnsBad++;
+			this->stats_failure++;
 			recv_packet.error = error;
 			request->OnError(&recv_packet);
 		}
 		else if (recv_packet.answers.empty())
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "No resource records returned");
-			ServerInstance->stats.DnsBad++;
-			recv_packet.error = ERROR_NO_RECORDS;
+			ServerInstance->Logs.Debug(MODNAME, "No resource records returned");
+			this->stats_failure++;
+			recv_packet.error = DNS::ERROR_NO_RECORDS;
 			request->OnError(&recv_packet);
 		}
 		else
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Lookup complete for " + request->question.name);
-			ServerInstance->stats.DnsGood++;
+			ServerInstance->Logs.Debug(MODNAME, "Lookup complete for {}", request->question.name);
+			this->stats_success++;
 			request->OnLookupComplete(&recv_packet);
 			this->AddCache(recv_packet);
 		}
 
-		ServerInstance->stats.Dns++;
+		this->stats_total++;
 
 		/* Request's destructor removes it from the request map */
 		delete request;
 	}
 
-	bool Tick(time_t now) CXX11_OVERRIDE
+	bool Tick() override
 	{
 		unsigned long expired = 0;
 		for (cache_map::iterator it = this->cache.begin(); it != this->cache.end(); )
 		{
-			const Query& query = it->second;
-			if (IsExpired(query, now))
+			const auto& query = it->second;
+			if (IsExpired(query))
 			{
 				expired++;
 				this->cache.erase(it++);
@@ -730,14 +774,14 @@ class MyManager : public Manager, public Timer, public EventHandler
 		}
 
 		if (expired)
-			ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "cache: purged %lu expired DNS entries", expired);
+			ServerInstance->Logs.Debug(MODNAME, "cache: purged {} expired DNS entries", expired);
 
 		return true;
 	}
 
-	void Rehash(const std::string& dnsserver, std::string sourceaddr, unsigned int sourceport)
+	void Rehash(const std::string& dnsserver, std::string sourceaddr, in_port_t sourceport)
 	{
-		irc::sockets::aptosa(dnsserver, DNS::PORT, myserver);
+		myserver.from_ip_port(dnsserver, DNS::PORT);
 
 		/* Initialize mastersocket */
 		Close();
@@ -747,56 +791,59 @@ class MyManager : public Manager, public Timer, public EventHandler
 		/* Have we got a socket? */
 		if (this->HasFd())
 		{
-			SocketEngine::SetReuse(s);
+			SocketEngine::SetOption<int>(s, SOL_SOCKET, SO_REUSEADDR, 1);
 			SocketEngine::NonBlocking(s);
 
-			irc::sockets::sockaddrs bindto;
 			if (sourceaddr.empty())
 			{
-				// set a sourceaddr for irc::sockets::aptosa() based on the servers af type
+				// set a sourceaddr based on the servers af type
 				if (myserver.family() == AF_INET)
 					sourceaddr = "0.0.0.0";
 				else if (myserver.family() == AF_INET6)
 					sourceaddr = "::";
 			}
-			irc::sockets::aptosa(sourceaddr, sourceport, bindto);
 
-			if (SocketEngine::Bind(this->GetFd(), bindto) < 0)
+			irc::sockets::sockaddrs bindto;
+			bindto.from_ip_port(sourceaddr, sourceport);
+
+			if (SocketEngine::Bind(this, bindto) < 0)
 			{
 				/* Failed to bind */
-				ServerInstance->Logs->Log(MODNAME, LOG_SPARSE, "Error binding dns socket - hostnames will NOT resolve");
+				ServerInstance->Logs.Critical(MODNAME, "Error binding dns socket - hostnames will NOT resolve");
 				SocketEngine::Close(this->GetFd());
 				this->SetFd(-1);
 			}
 			else if (!SocketEngine::AddFd(this, FD_WANT_POLL_READ | FD_WANT_NO_WRITE))
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_SPARSE, "Internal error starting DNS - hostnames will NOT resolve.");
+				ServerInstance->Logs.Critical(MODNAME, "Internal error starting DNS - hostnames will NOT resolve.");
 				SocketEngine::Close(this->GetFd());
 				this->SetFd(-1);
 			}
 
 			if (bindto.family() != myserver.family())
-				ServerInstance->Logs->Log(MODNAME, LOG_SPARSE, "Nameserver address family differs from source address family - hostnames might not resolve");
+				ServerInstance->Logs.Warning(MODNAME, "Nameserver address family differs from source address family - hostnames might not resolve");
 		}
 		else
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_SPARSE, "Error creating DNS socket - hostnames will NOT resolve");
+			ServerInstance->Logs.Critical(MODNAME, "Error creating DNS socket - hostnames will NOT resolve");
 		}
 	}
 };
 
-class ModuleDNS : public Module
+class ModuleDNS final
+	: public Module
+	, public Stats::EventListener
 {
 	MyManager manager;
 	std::string DNSServer;
 	std::string SourceIP;
-	unsigned int SourcePort;
+	in_port_t SourcePort = 0;
 
 	void FindDNSServer()
 	{
 #ifdef _WIN32
 		// attempt to look up their nameserver from the system
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "WARNING: <dns:server> not defined, attempting to find a working server in the system settings...");
+		ServerInstance->Logs.Normal(MODNAME, "WARNING: <dns:server> not defined, attempting to find a working server in the system settings...");
 
 		PFIXED_INFO pFixedInfo;
 		DWORD dwBufferSize = sizeof(FIXED_INFO);
@@ -820,15 +867,15 @@ class ModuleDNS : public Module
 
 			if (!DNSServer.empty())
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "<dns:server> set to '%s' as first active resolver in the system settings.", DNSServer.c_str());
+				ServerInstance->Logs.Normal(MODNAME, "<dns:server> set to '{}' as first active resolver in the system settings.", DNSServer);
 				return;
 			}
 		}
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "No viable nameserver found! Defaulting to nameserver '127.0.0.1'!");
+		ServerInstance->Logs.Warning(MODNAME, "No viable nameserver found! Defaulting to nameserver '127.0.0.1'!");
 #else
 		// attempt to look up their nameserver from /etc/resolv.conf
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "WARNING: <dns:server> not defined, attempting to find working server in /etc/resolv.conf...");
+		ServerInstance->Logs.Normal(MODNAME, "WARNING: <dns:server> not defined, attempting to find working server in /etc/resolv.conf...");
 
 		std::ifstream resolv("/etc/resolv.conf");
 
@@ -839,26 +886,28 @@ class ModuleDNS : public Module
 				resolv >> DNSServer;
 				if (DNSServer.find_first_not_of("0123456789.") == std::string::npos || DNSServer.find_first_not_of("0123456789ABCDEFabcdef:") == std::string::npos)
 				{
-					ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "<dns:server> set to '%s' as first resolver in /etc/resolv.conf.",DNSServer.c_str());
+					ServerInstance->Logs.Normal(MODNAME, "<dns:server> set to '{}' as first resolver in /etc/resolv.conf.", DNSServer);
 					return;
 				}
 			}
 		}
 
-		ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "/etc/resolv.conf contains no viable nameserver entries! Defaulting to nameserver '127.0.0.1'!");
+		ServerInstance->Logs.Warning(MODNAME, "/etc/resolv.conf contains no viable nameserver entries! Defaulting to nameserver '127.0.0.1'!");
 #endif
 		DNSServer = "127.0.0.1";
 	}
 
- public:
-	ModuleDNS() : manager(this)
-		, SourcePort(0)
+public:
+	ModuleDNS()
+		: Module(VF_CORE | VF_VENDOR, "Provides support for DNS lookups")
+		, Stats::EventListener(this)
+		, manager(this)
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("dns");
+		const auto& tag = ServerInstance->Config->ConfValue("dns");
 		if (!tag->getBool("enabled", true))
 		{
 			// Clear these so they get reset if DNS is enabled again.
@@ -876,19 +925,31 @@ class ModuleDNS : public Module
 		const std::string oldip = SourceIP;
 		SourceIP = tag->getString("sourceip");
 
-		const unsigned int oldport = SourcePort;
-		SourcePort = tag->getUInt("sourceport", 0, 0, UINT16_MAX);
+		const in_port_t oldport = SourcePort;
+		SourcePort = tag->getNum<in_port_t>("sourceport", 0);
 
 		if (DNSServer.empty())
 			FindDNSServer();
 
 		if (oldserver != DNSServer || oldip != SourceIP || oldport != SourcePort || !SourcePort)
 			this->manager.Rehash(DNSServer, SourceIP, SourcePort);
+
+		this->manager.timeout = tag->getDuration("timeout", 5, 1);
 	}
 
-	void OnUnloadModule(Module* mod) CXX11_OVERRIDE
+	ModResult OnStats(Stats::Context& stats) override
 	{
-		for (unsigned int i = 0; i <= MAX_REQUEST_ID; ++i)
+		if (stats.GetSymbol() == 'T')
+		{
+			stats.AddGenericRow(INSP_FORMAT("DNS requests: {} ({} succeeded, {} failed)",
+				manager.stats_total, manager.stats_success, manager.stats_failure));
+		}
+		return MOD_RES_PASSTHRU;
+	}
+
+	void OnUnloadModule(Module* mod) override
+	{
+		for (unsigned int i = 0; i <= DNS::MAX_REQUEST_ID; ++i)
 		{
 			DNS::Request* req = this->manager.requests[i];
 			if (!req)
@@ -896,19 +957,14 @@ class ModuleDNS : public Module
 
 			if (req->creator == mod)
 			{
-				Query rr(req->question);
-				rr.error = ERROR_UNLOADED;
+				DNS::Query rr(req->question);
+				rr.error = DNS::ERROR_UNLOADED;
 				req->OnError(&rr);
 
 				delete req;
 			}
 		}
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Provides support for DNS lookups", VF_CORE|VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleDNS)
