@@ -1,13 +1,12 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2017, 2020-2024 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012, 2014, 2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2017, 2020-2022, 2024-2025 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2014, 2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007-2008 Craig Edwards <brain@inspircd.org>
- *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -44,6 +43,7 @@ CmdResult CommandCommands::HandleLocal(LocalUser* user, const Params& parameters
 	std::vector<Numeric::Numeric> numerics;
 	numerics.reserve(ServerInstance->Parser.GetCommands().size());
 
+	const auto has_auspex = user->HasPrivPermission("servers/auspex");
 	for (const auto& [_, command] : ServerInstance->Parser.GetCommands())
 	{
 		// Don't show privileged commands to users without the privilege.
@@ -65,7 +65,7 @@ CmdResult CommandCommands::HandleLocal(LocalUser* user, const Params& parameters
 		// Only send this command to the user if:
 		// 1. It is usable by the caller.
 		// 2. The caller has the servers/auspex priv.
-		if (!usable && !user->HasPrivPermission("servers/auspex"))
+		if (!usable && !has_auspex)
 			continue;
 
 		Numeric::Numeric numeric(RPL_COMMANDS);
@@ -82,7 +82,7 @@ CmdResult CommandCommands::HandleLocal(LocalUser* user, const Params& parameters
 
 	// Sort alphabetically by command name.
 	std::sort(numerics.begin(), numerics.end(), [](const auto& lhs, const auto& rhs) {
-		return lhs.GetParams()[0] > rhs.GetParams()[0];
+		return lhs.GetParams()[0] < rhs.GetParams()[0];
 	});
 
 	for (const auto& numeric : numerics)
