@@ -1,12 +1,11 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2020-2022, 2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2020-2022, 2024-2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2015 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2013-2014, 2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2007 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -44,11 +43,12 @@ static const char* const lines[] = {
 	"                 November 2002 - Present",
 	" ",
 	"\002Core Developers\002:",
-	"    Matt Schatz (genius3000)        Sadie Powell (Sadie)",
+	"    Sadie Powell (Sadie)",
 	" ",
 	"\002Former Developers\002:",
-	"    Attila Molnar (Attila)          Daniel De Graaf (danieldg)",
-	"    Dennis Friis (peavey)           John Brooks (special)",
+	"    Adam (Adam-)                    Attila Molnar (Attila)",
+	"    Daniel De Graaf (danieldg)      Dennis Friis (peavey)",
+	"    John Brooks (special)           Matt Schatz (genius3000)",
 	"    Matt Smith (dz)                 Oliver Lupton (Om)",
 	"    Thomas Stagner (aquanight)      Uli Schlachter (psychon)",
 	" ",
@@ -57,7 +57,7 @@ static const char* const lines[] = {
 	"    Robin Burchell (w00t)",
 	" ",
 	"\002Active Contributors\002:",
-	"    Adam            progval",
+	"    progval",
 	" ",
 	"\002Former Contributors\002:",
 	"    Adremelech      Ankit           AnMaster        Bricker",
