@@ -1,7 +1,7 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2019-2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2019 Matt Schatz <genius3000@g3k.solutions>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -26,9 +26,6 @@
 /// $PackageInfo: require_system("debian~") libmaxminddb-dev pkg-config
 /// $PackageInfo: require_system("rhel~") libmaxminddb-devel pkgconfig
 
-#ifdef _WIN32
-# pragma comment(lib, "maxminddb.lib")
-#endif
 
 #include <maxminddb.h>
 
@@ -181,8 +178,8 @@ public:
 		MMDB_s mmdb;
 		int result = MMDB_open(file.c_str(), MMDB_MODE_MMAP, &mmdb);
 		if (result != MMDB_SUCCESS)
-			throw ModuleException(this, INSP_FORMAT("Unable to load the MaxMind database ({}): {}",
-				file, MMDB_strerror(result)));
+			throw ModuleException(this, "Unable to load the MaxMind database ({}): {}",
+				file, MMDB_strerror(result));
 
 		// Swap the new database with the old database.
 		std::swap(mmdb, geoapi.mmdb);
