@@ -1,14 +1,13 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2015 Daniel Vassdal <shutter@canternet.org>
  *   Copyright (C) 2013-2014 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2013, 2016-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013, 2016-2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2012 ChrisTX <xpipe@hotmail.de>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2009 Uli Schlachter <psychon@inspircd.org>
+ *   Copyright (C) 2009 Uli Schlachter <psychon@znc.in>
  *   Copyright (C) 2007, 2009 Dennis Friis <peavey@inspircd.org>
  *   Copyright (C) 2005, 2008-2009 Craig Edwards <brain@inspircd.org>
  *
@@ -25,45 +24,23 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+
 /// $CompilerFlags: execute("mysql_config --include" "MYSQL_CXXFLAGS")
 /// $LinkerFlags: execute("mysql_config --libs_r" "MYSQL_LDFLAGS" "-lmysqlclient")
 
+/// $PackageInfo: require_system("alpine") mariadb-dev
 /// $PackageInfo: require_system("arch") mariadb-libs
-/// $PackageInfo: require_system("centos") mysql-devel
 /// $PackageInfo: require_system("darwin") mysql-client
-/// $PackageInfo: require_system("debian") libmysqlclient-dev
-/// $PackageInfo: require_system("rocky") mysql-devel
-/// $PackageInfo: require_system("ubuntu") libmysqlclient-dev
+/// $PackageInfo: require_system("debian~") libmysqlclient-dev
+/// $PackageInfo: require_system("rhel~") mariadb-connector-c-devel
 
-#ifdef __GNUC__
-# pragma GCC diagnostic push
-#endif
 
-// Fix warnings about the use of commas at end of enumerator lists and long long
-// on C++03.
-#if defined __clang__
-# pragma clang diagnostic ignored "-Wc++11-extensions"
-# pragma clang diagnostic ignored "-Wc++11-long-long"
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wlong-long"
-# if (__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ >= 8))
-#  pragma GCC diagnostic ignored "-Wpedantic"
-# else
-#  pragma GCC diagnostic ignored "-pedantic"
-# endif
-#endif
+#include <mysql.h>
 
 #include "inspircd.h"
-#include <mysql.h>
 #include "modules/sql.h"
-
-#ifdef __GNUC__
-# pragma GCC diagnostic pop
-#endif
-
-#ifdef _WIN32
-# pragma comment(lib, "libmysql.lib")
-#endif
+#include "threadsocket.h"
+#include "utility/string.h"
 
 /* VERSION 3 API: With nonblocking (threaded) requests */
 
@@ -103,7 +80,17 @@ class SQLConnection;
 class MySQLresult;
 class DispatcherThread;
 
-struct QueryQueueItem
+namespace
+{
+	template<typename Numeric>
+	bool IsMySQLError(Numeric num)
+	{
+		// MySQL regularly returns -1 on error in unsigned fields.
+		return num == static_cast<Numeric>(-1);
+	}
+}
+
+struct QueryQueueItem final
 {
 	// An SQL database which this query is executed on.
 	SQLConnection* connection;
@@ -122,7 +109,7 @@ struct QueryQueueItem
 	}
 };
 
-struct ResultQueueItem
+struct ResultQueueItem final
 {
 	// An object which handles the result of the query.
 	SQL::Query* query;
@@ -143,52 +130,54 @@ typedef std::deque<ResultQueueItem> ResultQueue;
 
 /** MySQL module
  *  */
-class ModuleSQL : public Module
+class ModuleSQL final
+	: public Module
 {
- public:
-	DispatcherThread* Dispatcher;
+public:
+	DispatcherThread* Dispatcher = nullptr;
 	QueryQueue qq;       // MUST HOLD MUTEX
 	ResultQueue rq;      // MUST HOLD MUTEX
 	ConnMap connections; // main thread only
 
+	void init() override;
 	ModuleSQL();
-	void init() CXX11_OVERRIDE;
-	~ModuleSQL();
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE;
-	void OnUnloadModule(Module* mod) CXX11_OVERRIDE;
-	Version GetVersion() CXX11_OVERRIDE;
+	~ModuleSQL() override;
+	void ReadConfig(ConfigStatus& status) override;
+	void OnUnloadModule(Module* mod) override;
 };
 
-class DispatcherThread : public SocketThread
+class DispatcherThread final
+	: public SocketThread
 {
- private:
+private:
 	ModuleSQL* const Parent;
- public:
-	DispatcherThread(ModuleSQL* CreatorModule) : Parent(CreatorModule) { }
-	~DispatcherThread() { }
-	void Run() CXX11_OVERRIDE;
-	void OnNotify() CXX11_OVERRIDE;
+public:
+	DispatcherThread(ModuleSQL* CreatorModule)
+		: Parent(CreatorModule)
+	{
+	}
+	void OnStart() override;
+	void OnNotify() override;
 };
 
 /** Represents a mysql result set
  */
-class MySQLresult : public SQL::Result
+class MySQLresult final
+	: public SQL::Result
 {
- public:
+public:
 	SQL::Error err;
-	int currentrow;
-	int rows;
+	int currentrow = 0;
+	int rows = 0;
 	std::vector<std::string> colnames;
 	std::vector<SQL::Row> fieldlists;
 
-	MySQLresult(MYSQL_RES* res, int affected_rows)
+	MySQLresult(MYSQL_RES* res, unsigned long affected_rows)
 		: err(SQL::SUCCESS)
-		, currentrow(0)
-		, rows(0)
 	{
-		if (affected_rows >= 1)
+		if (affected_rows >= 1 && !IsMySQLError(affected_rows))
 		{
-			rows = affected_rows;
+			rows = int(affected_rows);
 			fieldlists.resize(rows);
 		}
 		unsigned int field_count = 0;
@@ -198,7 +187,7 @@ class MySQLresult : public SQL::Result
 			int n = 0;
 			while ((row = mysql_fetch_row(res)))
 			{
-				if (fieldlists.size() < (unsigned int)rows+1)
+				if (fieldlists.size() < (size_t)rows+1)
 				{
 					fieldlists.resize(fieldlists.size()+1);
 				}
@@ -213,9 +202,9 @@ class MySQLresult : public SQL::Result
 					{
 						std::string a = (fields[field_count].name ? fields[field_count].name : "");
 						if (row[field_count])
-							fieldlists[n].push_back(SQL::Field(row[field_count]));
+							fieldlists[n].emplace_back(row[field_count]);
 						else
-							fieldlists[n].push_back(SQL::Field());
+							fieldlists[n].emplace_back();
 						colnames.push_back(a);
 						field_count++;
 					}
@@ -229,23 +218,21 @@ class MySQLresult : public SQL::Result
 
 	MySQLresult(SQL::Error& e)
 		: err(e)
-		, currentrow(0)
-		, rows(0)
 	{
 
 	}
 
-	int Rows() CXX11_OVERRIDE
+	int Rows() override
 	{
 		return rows;
 	}
 
-	void GetCols(std::vector<std::string>& result) CXX11_OVERRIDE
+	void GetCols(std::vector<std::string>& result) override
 	{
 		result.assign(colnames.begin(), colnames.end());
 	}
 
-	bool HasColumn(const std::string& column, size_t& index) CXX11_OVERRIDE
+	bool HasColumn(const std::string& column, size_t& index) override
 	{
 		for (size_t i = 0; i < colnames.size(); ++i)
 		{
@@ -264,10 +251,10 @@ class MySQLresult : public SQL::Result
 		{
 			return fieldlists[row][column];
 		}
-		return SQL::Field();
+		return std::nullopt;
 	}
 
-	bool GetRow(SQL::Row& result) CXX11_OVERRIDE
+	bool GetRow(SQL::Row& result) override
 	{
 		if (currentrow < rows)
 		{
@@ -285,10 +272,11 @@ class MySQLresult : public SQL::Result
 
 /** Represents a connection to a mysql database
  */
-class SQLConnection : public SQL::Provider
+class SQLConnection final
+	: public SQL::Provider
 {
- private:
-	bool EscapeString(SQL::Query* query, const std::string& in, std::string& out)
+private:
+	bool EscapeString(SQL::Query* query, const std::string& in, std::string& out) const
 	{
 		// In the worst case each character may need to be encoded as using two bytes and one
 		// byte is the NUL terminator.
@@ -300,32 +288,31 @@ class SQLConnection : public SQL::Provider
 		// Unfortunately, someone genius decided that mysql_escape_string should return an
 		// unsigned type even though -1 is returned on error so checking whether an error
 		// happened is a bit cursed.
-		unsigned long escapedsize = mysql_escape_string(&buffer[0], in.c_str(), in.length());
-		if (escapedsize == static_cast<unsigned long>(-1))
+		unsigned long escapedsize = mysql_escape_string(buffer.data(), in.c_str(), in.length());
+		if (IsMySQLError(escapedsize))
 		{
-			SQL::Error err(SQL::QSEND_FAIL, InspIRCd::Format("%u: %s", mysql_errno(connection), mysql_error(connection)));
+			SQL::Error err(SQL::QSEND_FAIL, INSP_FORMAT("{}: {}", mysql_errno(connection), mysql_error(connection)));
 			query->OnError(err);
 			return false;
 		}
 
-		out.append(&buffer[0], escapedsize);
+		out.append(buffer.data(), escapedsize);
 		return true;
 	}
 
- public:
-	reference<ConfigTag> config;
-	MYSQL *connection;
-	Mutex lock;
+public:
+	std::shared_ptr<ConfigTag> config;
+	MYSQL* connection = nullptr;
+	std::mutex lock;
 
 	// This constructor creates an SQLConnection object with the given credentials, but does not connect yet.
-	SQLConnection(Module* p, ConfigTag* tag)
+	SQLConnection(Module* p, const std::shared_ptr<ConfigTag>& tag)
 		: SQL::Provider(p, tag->getString("id"))
 		, config(tag)
-		, connection(NULL)
 	{
 	}
 
-	~SQLConnection()
+	~SQLConnection() override
 	{
 		mysql_close(connection);
 	}
@@ -343,7 +330,7 @@ class SQLConnection : public SQL::Provider
 		connection = mysql_init(connection);
 
 		// Set the connection timeout.
-		unsigned int timeout = config->getDuration("timeout", 5, 1, 30);
+		unsigned int timeout = static_cast<unsigned int>(config->getDuration("timeout", 5, 1, 30));
 		mysql_options(connection, MYSQL_OPT_CONNECT_TIMEOUT, &timeout);
 
 		// Enable SSL if requested.
@@ -357,11 +344,26 @@ class SQLConnection : public SQL::Provider
 		const std::string user = config->getString("user");
 		const std::string pass = config->getString("pass");
 		const std::string dbname = config->getString("name");
-		unsigned int port = config->getUInt("port", 3306, 1, 65535);
-		if (!mysql_real_connect(connection, host.c_str(), user.c_str(), pass.c_str(), dbname.c_str(), port, NULL, CLIENT_IGNORE_SIGPIPE))
+
+		MYSQL* result;
+#if defined LIBMYSQL_VERSION_ID && LIBMYSQL_VERSION_ID > 80000
+		if (config->getBool("srv"))
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Unable to connect to the %s MySQL server: %s",
-				GetId().c_str(), mysql_error(connection));
+			result = mysql_real_connect_dns_srv(connection, host.c_str(), user.c_str(),
+				pass.c_str(), dbname.c_str(), CLIENT_IGNORE_SIGPIPE);
+		}
+		else
+#endif
+		{
+			auto port = config->getNum<unsigned int>("port", 3306, 1, 65535);
+			result = mysql_real_connect(connection, host.c_str(), user.c_str(), pass.c_str(),
+				dbname.c_str(), port, nullptr, CLIENT_IGNORE_SIGPIPE);
+		}
+
+		if (!result)
+		{
+			ServerInstance->Logs.Critical(MODNAME, "Unable to connect to the {} MySQL server: {}",
+				GetId(), mysql_error(connection));
 			return false;
 		}
 
@@ -369,8 +371,8 @@ class SQLConnection : public SQL::Provider
 		const std::string charset = config->getString("charset");
 		if (!charset.empty() && mysql_set_character_set(connection, charset.c_str()))
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Could not set character set for %s to \"%s\": %s",
-				GetId().c_str(), charset.c_str(), mysql_error(connection));
+			ServerInstance->Logs.Critical(MODNAME, "Could not set character set for {} to \"{}\": {}",
+				GetId(), charset, mysql_error(connection));
 			return false;
 		}
 
@@ -378,8 +380,8 @@ class SQLConnection : public SQL::Provider
 		const std::string initialquery = config->getString("initialquery");
 		if (!initialquery.empty() && mysql_real_query(connection, initialquery.data(), initialquery.length()))
 		{
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "Could not execute initial query \"%s\" for %s: %s",
-				initialquery.c_str(), name.c_str(), mysql_error(connection));
+			ServerInstance->Logs.Critical(MODNAME, "Could not execute initial query \"{}\" for {}: {}",
+				initialquery, name, mysql_error(connection));
 			return false;
 		}
 
@@ -388,7 +390,7 @@ class SQLConnection : public SQL::Provider
 
 	ModuleSQL* Parent()
 	{
-		return (ModuleSQL*)(Module*)creator;
+		return static_cast<ModuleSQL*>(creator.ptr());
 	}
 
 	MySQLresult* DoBlockingQuery(const std::string& query)
@@ -406,7 +408,7 @@ class SQLConnection : public SQL::Provider
 		{
 			/* XXX: See /usr/include/mysql/mysqld_error.h for a list of
 			 * possible error numbers and error messages */
-			SQL::Error e(SQL::QREPLY_FAIL, InspIRCd::Format("%u: %s", mysql_errno(connection), mysql_error(connection)));
+			SQL::Error e(SQL::QREPLY_FAIL, INSP_FORMAT("{}: {}", mysql_errno(connection), mysql_error(connection)));
 			return new MySQLresult(e);
 		}
 	}
@@ -418,29 +420,29 @@ class SQLConnection : public SQL::Provider
 		return true;
 	}
 
-	void Submit(SQL::Query* q, const std::string& qs) CXX11_OVERRIDE
+	void Submit(SQL::Query* q, const std::string& qs) override
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Executing MySQL query: " + qs);
+		ServerInstance->Logs.Debug(MODNAME, "Executing MySQL query: {}", qs);
 		Parent()->Dispatcher->LockQueue();
-		Parent()->qq.push_back(QueryQueueItem(q, qs, this));
+		Parent()->qq.emplace_back(q, qs, this);
 		Parent()->Dispatcher->UnlockQueueWakeup();
 	}
 
-	void Submit(SQL::Query* call, const std::string& q, const SQL::ParamList& p) CXX11_OVERRIDE
+	void Submit(SQL::Query* call, const std::string& q, const SQL::ParamList& p) override
 	{
 		std::string res;
 		unsigned int param = 0;
-		for(std::string::size_type i = 0; i < q.length(); i++)
+		for (const auto chr : q)
 		{
-			if (q[i] != '?')
-				res.push_back(q[i]);
+			if (chr != '?')
+				res.push_back(chr);
 			else if (param < p.size() && !EscapeString(call, p[param++], res))
 				return;
 		}
 		Submit(call, res);
 	}
 
-	void Submit(SQL::Query* call, const std::string& q, const SQL::ParamMap& p) CXX11_OVERRIDE
+	void Submit(SQL::Query* call, const std::string& q, const SQL::ParamMap& p) override
 	{
 		std::string res;
 		for(std::string::size_type i = 0; i < q.length(); i++)
@@ -464,33 +466,34 @@ class SQLConnection : public SQL::Provider
 	}
 };
 
-ModuleSQL::ModuleSQL()
-	: Dispatcher(NULL)
-{
-}
-
 void ModuleSQL::init()
 {
-	if (mysql_library_init(0, NULL, NULL))
-		throw ModuleException("Unable to initialise the MySQL library!");
+	if (mysql_library_init(0, nullptr, nullptr))
+		throw ModuleException(this, "Unable to initialise the MySQL library!");
+
+	ServerInstance->Logs.Normal(MODNAME, "Module was compiled against MySQL version {}.{}.{} and is running against version {}",
+		MYSQL_VERSION_ID / 10000, MYSQL_VERSION_ID / 100 % 100, MYSQL_VERSION_ID % 100, mysql_get_client_info());
 
 	Dispatcher = new DispatcherThread(this);
-	ServerInstance->Threads.Start(Dispatcher);
+	Dispatcher->Start();
+}
+
+ModuleSQL::ModuleSQL()
+	: Module(VF_VENDOR, "Provides the ability for SQL modules to query a MySQL database.")
+{
 }
 
 ModuleSQL::~ModuleSQL()
 {
 	if (Dispatcher)
 	{
-		Dispatcher->join();
+		Dispatcher->Stop();
 		Dispatcher->OnNotify();
 		delete Dispatcher;
 	}
 
-	for(ConnMap::iterator i = connections.begin(); i != connections.end(); i++)
-	{
-		delete i->second;
-	}
+	for (const auto& [_, connection] : connections)
+		delete connection;
 
 	mysql_library_end();
 }
@@ -498,18 +501,19 @@ ModuleSQL::~ModuleSQL()
 void ModuleSQL::ReadConfig(ConfigStatus& status)
 {
 	ConnMap conns;
-	ConfigTagList tags = ServerInstance->Config->ConfTags("database");
-	for(ConfigIter i = tags.first; i != tags.second; i++)
+
+	for (const auto& [_, tag] : ServerInstance->Config->ConfTags("database"))
 	{
-		if (!stdalgo::string::equalsci(i->second->getString("module"), "mysql"))
+		if (!insp::equalsci(tag->getString("module"), "mysql"))
 			continue;
-		std::string id = i->second->getString("id");
+
+		std::string id = tag->getString("id");
 		ConnMap::iterator curr = connections.find(id);
 		if (curr == connections.end())
 		{
-			SQLConnection* conn = new SQLConnection(this, i->second);
-			conns.insert(std::make_pair(id, conn));
-			ServerInstance->Modules->AddService(*conn);
+			auto* conn = new SQLConnection(this, tag);
+			conns.emplace(id, conn);
+			ServerInstance->Modules.AddService(*conn);
 		}
 		else
 		{
@@ -521,17 +525,18 @@ void ModuleSQL::ReadConfig(ConfigStatus& status)
 	// now clean up the deleted databases
 	Dispatcher->LockQueue();
 	SQL::Error err(SQL::BAD_DBID);
-	for(ConnMap::iterator i = connections.begin(); i != connections.end(); i++)
+
+	for (const auto& [_, connection] : connections)
 	{
-		ServerInstance->Modules->DelService(*i->second);
+		ServerInstance->Modules.DelService(*connection);
 		// it might be running a query on this database. Wait for that to complete
-		i->second->lock.Lock();
-		i->second->lock.Unlock();
+		connection->lock.lock();
+		connection->lock.unlock();
 		// now remove all active queries to this DB
 		for (size_t j = qq.size(); j > 0; j--)
 		{
 			size_t k = j - 1;
-			if (qq[k].connection == i->second)
+			if (qq[k].connection == connection)
 			{
 				qq[k].query->OnError(err);
 				delete qq[k].query;
@@ -539,7 +544,7 @@ void ModuleSQL::ReadConfig(ConfigStatus& status)
 			}
 		}
 		// finally, nuke the connection
-		delete i->second;
+		delete connection;
 	}
 	Dispatcher->UnlockQueue();
 	connections.swap(conns);
@@ -549,7 +554,7 @@ void ModuleSQL::OnUnloadModule(Module* mod)
 {
 	SQL::Error err(SQL::BAD_DBID);
 	Dispatcher->LockQueue();
-	unsigned int i = qq.size();
+	size_t i = qq.size();
 	while (i > 0)
 	{
 		i--;
@@ -559,8 +564,7 @@ void ModuleSQL::OnUnloadModule(Module* mod)
 			{
 				// need to wait until the query is done
 				// (the result will be discarded)
-				qq[i].connection->lock.Lock();
-				qq[i].connection->lock.Unlock();
+				qq[i].connection->lock.lock();
 			}
 			qq[i].query->OnError(err);
 			delete qq[i].query;
@@ -572,23 +576,18 @@ void ModuleSQL::OnUnloadModule(Module* mod)
 	Dispatcher->OnNotify();
 }
 
-Version ModuleSQL::GetVersion()
-{
-	return Version("Provides the ability for SQL modules to query a MySQL database.", VF_VENDOR);
-}
-
-void DispatcherThread::Run()
+void DispatcherThread::OnStart()
 {
 	this->LockQueue();
-	while (!this->GetExitFlag())
+	while (!this->IsStopping())
 	{
 		if (!Parent->qq.empty())
 		{
 			QueryQueueItem i = Parent->qq.front();
-			i.connection->lock.Lock();
+			i.connection->lock.lock();
 			this->UnlockQueue();
 			MySQLresult* res = i.connection->DoBlockingQuery(i.querystr);
-			i.connection->lock.Unlock();
+			i.connection->lock.unlock();
 
 			/*
 			 * At this point, the main thread could be working on:
@@ -600,7 +599,7 @@ void DispatcherThread::Run()
 			if (!Parent->qq.empty() && Parent->qq.front().query == i.query)
 			{
 				Parent->qq.pop_front();
-				Parent->rq.push_back(ResultQueueItem(i.query, res));
+				Parent->rq.emplace_back(i.query, res);
 				NotifyParent();
 			}
 			else
@@ -624,15 +623,15 @@ void DispatcherThread::OnNotify()
 {
 	// this could unlock during the dispatch, but OnResult isn't expected to take that long
 	this->LockQueue();
-	for(ResultQueue::iterator i = Parent->rq.begin(); i != Parent->rq.end(); i++)
+	for (const auto& item : Parent->rq)
 	{
-		MySQLresult* res = i->result;
+		MySQLresult* res = item.result;
 		if (res->err.code == SQL::SUCCESS)
-			i->query->OnResult(*res);
+			item.query->OnResult(*res);
 		else
-			i->query->OnError(res->err);
-		delete i->query;
-		delete i->result;
+			item.query->OnError(res->err);
+		delete item.query;
+		delete item.result;
 	}
 	Parent->rq.clear();
 	this->UnlockQueue();
