@@ -1,13 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2015 Daniel Vassdal <shutter@canternet.org>
- *   Copyright (C) 2013-2014, 2016-2019, 2023 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2013-2014, 2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2012, 2016 Adam <Adam@anope.org>
- *   Copyright (C) 2012 ChrisTX <xpipe@hotmail.de>
+ *   Copyright (C) 2014, 2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2013-2014, 2016-2017, 2019-2025 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2010 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2007, 2009 Craig Edwards <brain@inspircd.org>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
@@ -26,60 +23,38 @@
  */
 
 /// $CompilerFlags: find_compiler_flags("sqlite3")
-/// $LinkerFlags: find_linker_flags("sqlite3" "-lsqlite3")
+/// $LinkerFlags: find_linker_flags("sqlite3")
 
+/// $PackageInfo: require_system("alpine") pkgconf sqlite-dev
 /// $PackageInfo: require_system("arch") pkgconf sqlite
-/// $PackageInfo: require_system("centos") pkgconfig sqlite-devel
 /// $PackageInfo: require_system("darwin") pkg-config sqlite
-/// $PackageInfo: require_system("debian") libsqlite3-dev pkg-config
-/// $PackageInfo: require_system("rocky") pkgconfig sqlite-devel
-/// $PackageInfo: require_system("ubuntu") libsqlite3-dev pkg-config
+/// $PackageInfo: require_system("debian~") libsqlite3-dev pkg-config
+/// $PackageInfo: require_system("rhel~") pkgconfig sqlite-devel
 
 #include "inspircd.h"
 #include "modules/sql.h"
-
-#ifdef __GNUC__
-# pragma GCC diagnostic push
-#endif
-
-// Fix warnings about the use of `long long` on C++03.
-#if defined __clang__
-# pragma clang diagnostic ignored "-Wc++11-long-long"
-#elif defined __GNUC__
-# pragma GCC diagnostic ignored "-Wlong-long"
-#endif
+#include "utility/string.h"
 
 #include <sqlite3.h>
 
-#ifdef __GNUC__
-# pragma GCC diagnostic pop
-#endif
-
-#ifdef _WIN32
-# pragma comment(lib, "sqlite3.lib")
-#endif
-
 class SQLConn;
 typedef insp::flat_map<std::string, SQLConn*> ConnMap;
 
-class SQLite3Result : public SQL::Result
+class SQLite3Result final
+	: public SQL::Result
 {
- public:
-	int currentrow;
-	int rows;
+public:
+	int currentrow = 0;
+	int rows = 0;
 	std::vector<std::string> columns;
 	std::vector<SQL::Row> fieldlists;
 
-	SQLite3Result() : currentrow(0), rows(0)
-	{
-	}
-
-	int Rows() CXX11_OVERRIDE
+	int Rows() override
 	{
 		return rows;
 	}
 
-	bool GetRow(SQL::Row& result) CXX11_OVERRIDE
+	bool GetRow(SQL::Row& result) override
 	{
 		if (currentrow < rows)
 		{
@@ -94,12 +69,12 @@ class SQLite3Result : public SQL::Result
 		}
 	}
 
-	void GetCols(std::vector<std::string>& result) CXX11_OVERRIDE
+	void GetCols(std::vector<std::string>& result) override
 	{
 		result.assign(columns.begin(), columns.end());
 	}
 
-	bool HasColumn(const std::string& column, size_t& index) CXX11_OVERRIDE
+	bool HasColumn(const std::string& column, size_t& index) override
 	{
 		for (size_t i = 0; i < columns.size(); ++i)
 		{
@@ -113,27 +88,28 @@ class SQLite3Result : public SQL::Result
 	}
 };
 
-class SQLConn : public SQL::Provider
+class SQLConn final
+	: public SQL::Provider
 {
 	sqlite3* conn;
-	reference<ConfigTag> config;
+	std::shared_ptr<ConfigTag> config;
 
- public:
-	SQLConn(Module* Parent, ConfigTag* tag)
+public:
+	SQLConn(Module* Parent, const std::shared_ptr<ConfigTag>& tag)
 		: SQL::Provider(Parent, tag->getString("id"))
 		, config(tag)
 	{
 		std::string host = tag->getString("hostname");
-		if (sqlite3_open_v2(host.c_str(), &conn, SQLITE_OPEN_READWRITE, 0) != SQLITE_OK)
+		if (sqlite3_open_v2(host.c_str(), &conn, SQLITE_OPEN_READWRITE, nullptr) != SQLITE_OK)
 		{
 			// Even in case of an error conn must be closed
 			sqlite3_close(conn);
-			conn = NULL;
-			ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "WARNING: Could not open DB with id: " + tag->getString("id"));
+			conn = nullptr;
+			ServerInstance->Logs.Critical(MODNAME, "WARNING: Could not open DB with id: " + tag->getString("id"));
 		}
 	}
 
-	~SQLConn()
+	~SQLConn() override
 	{
 		if (conn)
 		{
@@ -144,9 +120,16 @@ class SQLConn : public SQL::Provider
 
 	void Query(SQL::Query* query, const std::string& q)
 	{
+		if (!conn)
+		{
+			SQL::Error error(SQL::BAD_CONN, "Database connection is null");
+			query->OnError(error);
+			return;
+		}
+
 		SQLite3Result res;
-		sqlite3_stmt *stmt;
-		int err = sqlite3_prepare_v2(conn, q.c_str(), q.length(), &stmt, NULL);
+		sqlite3_stmt* stmt;
+		int err = sqlite3_prepare_v2(conn, q.c_str(), static_cast<int>(q.length()), &stmt, nullptr);
 		if (err != SQLITE_OK)
 		{
 			SQL::Error error(SQL::QSEND_FAIL, sqlite3_errmsg(conn));
@@ -159,7 +142,7 @@ class SQLConn : public SQL::Provider
 		{
 			res.columns[i] = sqlite3_column_name(stmt, i);
 		}
-		while (1)
+		while (true)
 		{
 			err = sqlite3_step(stmt);
 			if (err == SQLITE_ROW)
@@ -171,7 +154,7 @@ class SQLConn : public SQL::Provider
 				{
 					const char* txt = (const char*)sqlite3_column_text(stmt, i);
 					if (txt)
-						res.fieldlists[res.rows][i] = SQL::Field(txt);
+						res.fieldlists[res.rows][i] = txt;
 				}
 				res.rows++;
 			}
@@ -190,21 +173,21 @@ class SQLConn : public SQL::Provider
 		sqlite3_finalize(stmt);
 	}
 
-	void Submit(SQL::Query* query, const std::string& q) CXX11_OVERRIDE
+	void Submit(SQL::Query* query, const std::string& q) override
 	{
-		ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Executing SQLite3 query: " + q);
+		ServerInstance->Logs.Debug(MODNAME, "Executing SQLite3 query: {}", q);
 		Query(query, q);
 		delete query;
 	}
 
-	void Submit(SQL::Query* query, const std::string& q, const SQL::ParamList& p) CXX11_OVERRIDE
+	void Submit(SQL::Query* query, const std::string& q, const SQL::ParamList& p) override
 	{
 		std::string res;
 		unsigned int param = 0;
-		for(std::string::size_type i = 0; i < q.length(); i++)
+		for (const auto chr : q)
 		{
-			if (q[i] != '?')
-				res.push_back(q[i]);
+			if (chr != '?')
+				res.push_back(chr);
 			else
 			{
 				if (param < p.size())
@@ -218,7 +201,7 @@ class SQLConn : public SQL::Provider
 		Submit(query, res);
 	}
 
-	void Submit(SQL::Query* query, const std::string& q, const SQL::ParamMap& p) CXX11_OVERRIDE
+	void Submit(SQL::Query* query, const std::string& q, const SQL::ParamMap& p) override
 	{
 		std::string res;
 		for(std::string::size_type i = 0; i < q.length(); i++)
@@ -246,44 +229,52 @@ class SQLConn : public SQL::Provider
 	}
 };
 
-class ModuleSQLite3 : public Module
+class ModuleSQLite3 final
+	: public Module
 {
+private:
 	ConnMap conns;
 
- public:
-	~ModuleSQLite3()
+public:
+	ModuleSQLite3()
+		: Module(VF_VENDOR, "Provides the ability for SQL modules to query a SQLite 3 database.")
+	{
+	}
+
+	~ModuleSQLite3() override
 	{
 		ClearConns();
 	}
 
+	void init() override
+	{
+		ServerInstance->Logs.Normal(MODNAME, "Module was compiled against SQLite version {} and is running against version {}",
+			SQLITE_VERSION, sqlite3_libversion());
+	}
+
 	void ClearConns()
 	{
-		for(ConnMap::iterator i = conns.begin(); i != conns.end(); i++)
+		for (const auto& [_, conn] : conns)
 		{
-			SQLConn* conn = i->second;
-			ServerInstance->Modules->DelService(*conn);
+			ServerInstance->Modules.DelService(*conn);
 			delete conn;
 		}
 		conns.clear();
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
 		ClearConns();
-		ConfigTagList tags = ServerInstance->Config->ConfTags("database");
-		for(ConfigIter i = tags.first; i != tags.second; i++)
-		{
-			if (!stdalgo::string::equalsci(i->second->getString("module"), "sqlite"))
-				continue;
-			SQLConn* conn = new SQLConn(this, i->second);
-			conns.insert(std::make_pair(i->second->getString("id"), conn));
-			ServerInstance->Modules->AddService(*conn);
-		}
-	}
 
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Provides the ability for SQL modules to query a SQLite 3 database.", VF_VENDOR);
+		for (const auto& [_, tag] : ServerInstance->Config->ConfTags("database"))
+		{
+			if (!insp::equalsci(tag->getString("module"), "sqlite"))
+				continue;
+
+			auto* conn = new SQLConn(this, tag);
+			conns.emplace(tag->getString("id"), conn);
+			ServerInstance->Modules.AddService(*conn);
+		}
 	}
 };
 
