@@ -5,8 +5,8 @@
  *   Copyright (C) 2020 Matt Schatz <genius3000@g3k.solutions>
  *   Copyright (C) 2013-2014, 2016-2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Daniel Vassdal <shutter@canternet.org>
+ *   Copyright (C) 2013 Adam <Adam@anope.org>
  *   Copyright (C) 2012-2017 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2012-2013, 2016 Adam <Adam@anope.org>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2012 ChrisTX <xpipe@hotmail.de>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
@@ -58,10 +58,6 @@
 # define GNUTLS_AUTO_DH
 #endif
 
-#ifdef _WIN32
-# pragma comment(lib, "libgnutls-30.lib")
-#endif
-
 static Module* thismod;
 
 namespace GnuTLS
@@ -292,7 +288,8 @@ namespace GnuTLS
 			if (ret < 0)
 			{
 				// gnutls did not understand the user supplied string
-				throw Exception("Unable to initialize priorities to \"" + priorities + "\": " + gnutls_strerror(ret) + " Syntax error at position " + ConvToStr((unsigned int) (prioerror - priocstr)));
+				throw Exception(INSP_FORMAT("Unable to initialize priorities to \"{}\": {} Syntax error at position {}.",
+					priorities, gnutls_strerror(ret), static_cast<size_t>(prioerror - priocstr)));
 			}
 		}
 
@@ -791,7 +788,7 @@ private:
 		else if (certinfo->activation >= ServerInstance->Time())
 		{
 			certinfo->error = INSP_FORMAT("Certificate not active for {} (on {})",
-				Duration::ToString(certinfo->activation - ServerInstance->Time()),
+				Duration::ToLongString(certinfo->activation - ServerInstance->Time(), true),
 				Time::ToString(certinfo->activation));
 		}
 
@@ -804,7 +801,7 @@ private:
 		else if (certinfo->expiration <= ServerInstance->Time())
 		{
 			certinfo->error = INSP_FORMAT("Certificate expired {} ago (on {})",
-				Duration::ToString(ServerInstance->Time() - certinfo->expiration),
+				Duration::ToLongString(ServerInstance->Time() - certinfo->expiration, true),
 				Time::ToString(certinfo->expiration));
 		}
 
