@@ -51,8 +51,6 @@
 
 #ifdef _WIN32
 # define timegm _mkgmtime
-# pragma comment(lib, "libcrypto.lib")
-# pragma comment(lib, "libssl.lib")
 #endif
 
 #if OPENSSL_VERSION_NUMBER < 0x10101000L
@@ -247,6 +245,11 @@ namespace OpenSSL
 			}
 		}
 
+		void SetSecurityLevel(int securitylevel)
+		{
+			SSL_CTX_set_security_level(ctx, securitylevel);
+		}
+
 		long GetDefaultContextOptions() const
 		{
 			return ctx_options;
@@ -419,6 +422,10 @@ namespace OpenSSL
 			SetContextOptions("server", tag, ctx);
 			SetContextOptions("client", tag, clientctx);
 
+			const auto securitylevel = tag->getNum<int>("securitylevel", 0, 0, 10);
+			if (securitylevel)
+				ctx.SetSecurityLevel(securitylevel);
+
 			/* Load our keys and certificates
 			 * NOTE: OpenSSL's error logging API sucks, don't blame us for this clusterfuck.
 			 */
@@ -635,7 +642,7 @@ private:
 		if (activated != -1 && activated != 0)
 		{
 			certinfo->error = INSP_FORMAT("Certificate not active for {} (on {})",
-				Duration::ToString(certinfo->activation - ServerInstance->Time()),
+				Duration::ToLongString(certinfo->activation - ServerInstance->Time(), true),
 				Time::ToString(certinfo->activation));
 		}
 
@@ -643,7 +650,7 @@ private:
 		if (expired != 0 && expired != 1)
 		{
 			certinfo->error = INSP_FORMAT("Certificate expired {} ago (on {})",
-				Duration::ToString(ServerInstance->Time() - certinfo->expiration),
+				Duration::ToLongString(ServerInstance->Time() - certinfo->expiration, true),
 				Time::ToString(certinfo->expiration));
 		}
 
