@@ -1,7 +1,8 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2017, 2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2021 Dominic Hamon
+ *   Copyright (C) 2019-2023 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Daniel Vassdal <shutter@canternet.org>
  *   Copyright (C) 2012-2014 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
@@ -23,39 +24,39 @@
 
 #include "inspircd.h"
 
-static std::bitset<UCHAR_MAX + 1> allowedmap;
+static CharState allowedmap;
 
-class NewIsChannelHandler
+class NewIsChannelHandler final
 {
- public:
-	static bool Call(const std::string&);
+public:
+	static bool Call(const std::string_view&);
 };
 
-bool NewIsChannelHandler::Call(const std::string& channame)
+bool NewIsChannelHandler::Call(const std::string_view& channame)
 {
-	if (channame.empty() || channame.length() > ServerInstance->Config->Limits.ChanMax || channame[0] != '#')
+	if (channame.empty() || channame.length() > ServerInstance->Config->Limits.MaxChannel || !ServerInstance->Channels.IsPrefix(channame[0]))
 		return false;
 
-	for (std::string::const_iterator c = channame.begin(); c != channame.end(); ++c)
+	for (const auto chr : channame)
 	{
-		unsigned char i = static_cast<unsigned char>(*c);
-		if (!allowedmap[i])
+		if (!allowedmap[static_cast<unsigned char>(chr)])
 			return false;
 	}
 
 	return true;
 }
 
-class ModuleChannelNames : public Module
+class ModuleChannelNames final
+	: public Module
 {
-	TR1NS::function<bool(const std::string&)> rememberer;
-	bool badchan;
+	std::function<bool(const std::string_view&)> rememberer;
+	bool badchan = false;
 	ChanModeReference permchannelmode;
 
- public:
+public:
 	ModuleChannelNames()
-		: rememberer(ServerInstance->IsChannel)
-		, badchan(false)
+		: Module(VF_VENDOR, "Allows the server administrator to define what characters are allowed in channel names.")
+		, rememberer(ServerInstance->Channels.IsChannel)
 		, permchannelmode(this, "permanent")
 	{
 	}
@@ -65,20 +66,20 @@ class ModuleChannelNames : public Module
 		Modes::ChangeList removepermchan;
 
 		badchan = true;
-		const chan_hash& chans = ServerInstance->GetChans();
-		for (chan_hash::const_iterator i = chans.begin(); i != chans.end(); )
+		const ChannelMap& chans = ServerInstance->Channels.GetChans();
+		for (ChannelMap::const_iterator i = chans.begin(); i != chans.end(); )
 		{
 			Channel* c = i->second;
 			// Move iterator before we begin kicking
 			++i;
-			if (ServerInstance->IsChannel(c->name))
+			if (ServerInstance->Channels.IsChannel(c->name))
 				continue; // The name of this channel is still valid
 
-			if (c->IsModeSet(permchannelmode) && c->GetUserCounter())
+			if (c->IsModeSet(permchannelmode) && !c->GetUsers().empty())
 			{
 				removepermchan.clear();
 				removepermchan.push_remove(*permchannelmode);
-				ServerInstance->Modes->Process(ServerInstance->FakeClient, c, NULL, removepermchan);
+				ServerInstance->Modes.Process(ServerInstance->FakeClient, c, nullptr, removepermchan);
 			}
 
 			Channel::MemberMap& users = c->userlist;
@@ -97,9 +98,9 @@ class ModuleChannelNames : public Module
 		badchan = false;
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("channames");
+		const auto& tag = ServerInstance->Config->ConfValue("channames");
 		std::string denyToken = tag->getString("denyrange");
 		std::string allowToken = tag->getString("allowrange");
 
@@ -111,12 +112,12 @@ class ModuleChannelNames : public Module
 		allowedmap.set();
 
 		irc::portparser denyrange(denyToken, false);
-		int denyno = -1;
+		long denyno = -1;
 		while (0 != (denyno = denyrange.GetToken()))
 			allowedmap[denyno & UCHAR_MAX] = false;
 
 		irc::portparser allowrange(allowToken, false);
-		int allowno = -1;
+		long allowno = -1;
 		while (0 != (allowno = allowrange.GetToken()))
 			allowedmap[allowno & UCHAR_MAX] = true;
 
@@ -124,31 +125,27 @@ class ModuleChannelNames : public Module
 		allowedmap[0x20] = false; // ' '
 		allowedmap[0x2C] = false; // ','
 
-		ServerInstance->IsChannel = NewIsChannelHandler::Call;
+		ServerInstance->Channels.IsChannel = NewIsChannelHandler::Call;
 		ValidateChans();
 	}
 
-	void OnUserKick(User* source, Membership* memb, const std::string &reason, CUList& except_list) CXX11_OVERRIDE
+	void OnUserKick(User* source, Membership* memb, const std::string& reason, CUList& except_list) override
 	{
 		if (badchan)
 		{
-			const Channel::MemberMap& users = memb->chan->GetUsers();
-			for (Channel::MemberMap::const_iterator i = users.begin(); i != users.end(); ++i)
-				if (i->first != memb->user)
-					except_list.insert(i->first);
+			for (const auto& [user, _] : memb->chan->GetUsers())
+			{
+				if (user != memb->user)
+					except_list.insert(user);
+			}
 		}
 	}
 
-	CullResult cull() CXX11_OVERRIDE
+	Cullable::Result Cull() override
 	{
-		ServerInstance->IsChannel = rememberer;
+		ServerInstance->Channels.IsChannel = rememberer;
 		ValidateChans();
-		return Module::cull();
-	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows the server administrator to define what characters are allowed in channel names.", VF_VENDOR);
+		return Module::Cull();
 	}
 };
 
