@@ -2,12 +2,10 @@
  * InspIRCd -- Internet Relay Chat Daemon
  *
  *   Copyright (C) 2018 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2013, 2017-2018, 2020-2021 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2012, 2015-2016 Attila Molnar <attilamolnar@hush.com>
- *   Copyright (C) 2010 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2013, 2018-2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2012, 2015 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
- *   Copyright (C) 2007 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007 Dennis Friis <peavey@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
@@ -25,9 +23,10 @@
 
 
 #include "inspircd.h"
+#include "extension.h"
 #include "modules/whois.h"
 
-struct CustomTitle
+struct CustomTitle final
 {
 	const std::string name;
 	const std::string password;
@@ -48,123 +47,112 @@ struct CustomTitle
 
 	bool MatchUser(User* user) const
 	{
-		const std::string userHost = user->ident + "@" + user->GetRealHost();
-		const std::string userIP = user->ident + "@" + user->GetIPString();
-		return InspIRCd::MatchMask(host, userHost, userIP);
+		return InspIRCd::MatchMask(host, user->GetRealUserHost(), user->GetUserAddress());
 	}
 
-	bool CheckPass(User* user, const std::string& pass) const
+	bool CheckPass(const std::string& pass) const
 	{
-		return ServerInstance->PassCompare(user, password, pass, hash);
+		return InspIRCd::CheckPassword(password, hash, pass);
 	}
 };
 
 typedef std::multimap<std::string, CustomTitle> CustomVhostMap;
-typedef std::pair<CustomVhostMap::iterator, CustomVhostMap::iterator> MatchingConfigs;
 
-/** Handle /TITLE
- */
-class CommandTitle : public Command
+class CommandTitle final
+	: public Command
 {
- public:
+public:
 	StringExtItem ctitle;
 	CustomVhostMap configs;
 
-	CommandTitle(Module* Creator) : Command(Creator,"TITLE", 2),
-		ctitle("ctitle", ExtensionItem::EXT_USER, Creator)
+	CommandTitle(Module* Creator)
+		: Command(Creator, "TITLE", 2)
+		, ctitle(Creator, "ctitle", ExtensionType::USER, true)
 	{
-		syntax = "<username> <password>";
+		syntax = { "<username> <password>" };
 	}
 
-	CmdResult Handle(User* user, const Params& parameters) CXX11_OVERRIDE
+	CmdResult Handle(User* user, const Params& parameters) override
 	{
-		MatchingConfigs matching = configs.equal_range(parameters[0]);
-
-		for (MatchingConfigs::first_type i = matching.first; i != matching.second; ++i)
+		for (const auto& [_, config] : insp::equal_range(configs, parameters[0]))
 		{
-			CustomTitle config = i->second;
-			if (config.MatchUser(user) && config.CheckPass(user, parameters[1]))
+			if (config.MatchUser(user) && config.CheckPass(parameters[1]))
 			{
-				ctitle.set(user, config.title);
-
-				ServerInstance->PI->SendMetaData(user, "ctitle", config.title);
+				ctitle.Set(user, config.title);
 
 				if (!config.vhost.empty())
 					user->ChangeDisplayedHost(config.vhost);
 
 				user->WriteNotice("Custom title set to '" + config.title + "'");
 
-				return CMD_SUCCESS;
+				return CmdResult::SUCCESS;
 			}
 		}
 
 		user->WriteNotice("Invalid title credentials");
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 	}
 
 };
 
-class ModuleCustomTitle : public Module, public Whois::LineEventListener
+class ModuleCustomTitle final
+	: public Module
+	, public Whois::LineEventListener
 {
+private:
 	CommandTitle cmd;
 
- public:
+public:
 	ModuleCustomTitle()
-		: Whois::LineEventListener(this)
+		: Module(VF_VENDOR | VF_OPTCOMMON, "Allows the server administrator to define accounts which can grant a custom title in /WHOIS and an optional virtual host.")
+		, Whois::LineEventListener(this)
 		, cmd(this)
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTagList tags = ServerInstance->Config->ConfTags("title");
 		CustomVhostMap newtitles;
-		for (ConfigIter i = tags.first; i != tags.second; ++i)
+		for (const auto& [_, tag] : ServerInstance->Config->ConfTags("title"))
 		{
-			reference<ConfigTag> tag = i->second;
 			std::string name = tag->getString("name", "", 1);
 			if (name.empty())
-				throw ModuleException("<title:name> is empty at " + tag->getTagLocation());
+				throw ModuleException(this, "<title:name> is empty at " + tag->source.str());
 
 			std::string pass = tag->getString("password");
 			if (pass.empty())
-				throw ModuleException("<title:password> is empty at " + tag->getTagLocation());
+				throw ModuleException(this, "<title:password> is empty at " + tag->source.str());
 
 			const std::string hash = tag->getString("hash", "plaintext", 1);
-			if (stdalgo::string::equalsci(hash, "plaintext"))
+			if (insp::equalsci(hash, "plaintext"))
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEFAULT, "<title> tag for %s at %s contains an plain text password, this is insecure!",
-					name.c_str(), tag->getTagLocation().c_str());
+				ServerInstance->Logs.Normal(MODNAME, "<title> tag for {} at {} contains an plain text password, this is insecure!",
+					name, tag->source.str());
 			}
 
 			std::string host = tag->getString("host", "*@*", 1);
 			std::string title = tag->getString("title");
 			std::string vhost = tag->getString("vhost");
 			CustomTitle config(name, pass, hash, host, title, vhost);
-			newtitles.insert(std::make_pair(name, config));
+			newtitles.emplace(name, config);
 		}
 		cmd.configs.swap(newtitles);
 	}
 
 	// :kenny.chatspike.net 320 Brain Azhrarn :is getting paid to play games.
-	ModResult OnWhoisLine(Whois::Context& whois, Numeric::Numeric& numeric) CXX11_OVERRIDE
+	ModResult OnWhoisLine(Whois::Context& whois, Numeric::Numeric& numeric) override
 	{
 		/* We use this and not OnWhois because this triggers for remote, too */
 		if (numeric.GetNumeric() == RPL_WHOISSERVER)
 		{
 			/* Insert our numeric before 312 */
-			const std::string* ctitle = cmd.ctitle.get(whois.GetTarget());
+			const std::string* ctitle = cmd.ctitle.Get(whois.GetTarget());
 			if (ctitle && !ctitle->empty())
 				whois.SendLine(RPL_WHOISSPECIAL, *ctitle);
 		}
 		/* Don't block anything */
 		return MOD_RES_PASSTHRU;
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows the server administrator to define accounts which can grant a custom title in /WHOIS and an optional virtual host.", VF_OPTCOMMON | VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleCustomTitle)
