@@ -1,9 +1,9 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019-2020 Matt Schatz <genius3000@g3k.solutions>
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
- *   Copyright (C) 2018-2019, 2021, 2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2018-2019, 2021-2023 Sadie Powell <sadie@witchery.services>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -25,7 +25,7 @@
 
 enum
 {
-	// The SSL TLV flag for a client being connected over SSL.
+	// The TLV flag for a client being connected over TLS.
 	PP2_CLIENT_SSL = 0x01,
 
 	// The family for TCP over IPv4.
@@ -55,7 +55,7 @@ enum
 	// The minimum length of a Type-Length-Value entry.
 	PP2_TLV_LENGTH = 3,
 
-	// The identifier for a SSL TLV entry.
+	// The identifier for a TLS TLV entry.
 	PP2_TYPE_SSL = 0x20,
 
 	// The minimum length of a PP2_TYPE_SSL TLV entry.
@@ -92,7 +92,7 @@ enum HAProxyCommand
 	HPC_PROXY = 0x01
 };
 
-struct HAProxyHeader
+struct HAProxyHeader final
 {
 	// The signature used to identify the HAProxy protocol.
 	uint8_t signature[PP2_SIGNATURE_LENGTH];
@@ -107,34 +107,36 @@ struct HAProxyHeader
 	uint16_t length;
 };
 
-class HAProxyHookProvider : public IOHookProvider
+class HAProxyHookProvider final
+	: public IOHookProvider
 {
- private:
+private:
 	UserCertificateAPI sslapi;
 
- public:
+public:
 	HAProxyHookProvider(Module* mod)
 		: IOHookProvider(mod, "haproxy", IOHookProvider::IOH_UNKNOWN, true)
 		, sslapi(mod)
 	{
 	}
 
-	void OnAccept(StreamSocket* sock, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server) CXX11_OVERRIDE;
+	void OnAccept(StreamSocket* sock, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server) override;
 
-	void OnConnect(StreamSocket* sock) CXX11_OVERRIDE
+	void OnConnect(StreamSocket* sock) override
 	{
 		// We don't need to implement this.
 	}
 };
 
 // The signature for a HAProxy PROXY protocol header.
-static const char proxy_signature[13] = "\x0D\x0A\x0D\x0A\x00\x0D\x0A\x51\x55\x49\x54\x0A";
+static constexpr char proxy_signature[13] = "\x0D\x0A\x0D\x0A\x00\x0D\x0A\x51\x55\x49\x54\x0A";
 
-class HAProxyHook : public IOHookMiddle
+class HAProxyHook final
+	: public IOHookMiddle
 {
- private:
+private:
 	// The length of the address section.
-	uint16_t address_length;
+	uint16_t address_length = 0;
 
 	// The endpoint the client is connecting from.
 	irc::sockets::sockaddrs client;
@@ -145,11 +147,11 @@ class HAProxyHook : public IOHookMiddle
 	// The endpoint the client is connected to.
 	irc::sockets::sockaddrs server;
 
-	// The API for interacting with user SSL internals.
+	// The API for interacting with user TLS internals.
 	UserCertificateAPI& sslapi;
 
 	// The current state of the PROXY parser.
-	HAProxyState state;
+	HAProxyState state = HPS_WAITING_FOR_HEADER;
 
 	size_t ReadProxyTLV(StreamSocket* sock, size_t start_index, uint16_t buffer_length)
 	{
@@ -183,10 +185,10 @@ class HAProxyHook : public IOHookMiddle
 
 	bool ReadProxyTLVSSL(StreamSocket* sock, size_t start_index, uint16_t buffer_length)
 	{
-		// A SSL TLV must at least consist of client info (uint8_t) and verification info (uint32_t).
+		// A TLS TLV must at least consist of client info (uint8_t) and verification info (uint32_t).
 		if (buffer_length < PP2_TYPE_SSL_LENGTH)
 		{
-			sock->SetError("Truncated HAProxy PROXY SSL TLV");
+			sock->SetError("Truncated HAProxy PROXY TLS TLV");
 			return false;
 		}
 
@@ -200,16 +202,16 @@ class HAProxyHook : public IOHookMiddle
 		if (!sslapi)
 			return true;
 
-		// If the client is not connecting via TLS (SSL) the rest of this TLV is irrelevant.
+		// If the client is not connecting via TLS the rest of this TLV is irrelevant.
 		std::string& recvq = GetRecvQ();
 		if ((recvq[start_index] & PP2_CLIENT_SSL) == 0)
 			return true;
 
 		// Create a fake ssl_cert for the user. Ideally we should use the user's
-		// TLS (SSL) client certificate here but as of 2018-10-16 this is not forwarded
+		// TLS client certificate here but as of 2018-10-16 this is not forwarded
 		// by HAProxy.
-		ssl_cert* cert = new ssl_cert;
-		cert->error = "HAProxy does not forward client TLS (SSL) certificates";
+		auto* cert = new ssl_cert();
+		cert->error = "HAProxy does not forward client TLS certificates";
 		cert->invalid = true;
 		cert->revoked = true;
 		cert->trusted = false;
@@ -272,7 +274,7 @@ class HAProxyHook : public IOHookMiddle
 						break;
 				}
 
-				if (!sock->OnSetEndPoint(server, client))
+				if (!sock->OnChangeLocalSocketAddress(server) || !sock->OnChangeRemoteSocketAddress(client))
 					return -1;
 
 				// Parse any available TLVs.
@@ -374,29 +376,27 @@ class HAProxyHook : public IOHookMiddle
 		return ReadProxyAddress(sock, destrecvq);
 	}
 
- public:
-	HAProxyHook(IOHookProvider* Prov, StreamSocket* sock, UserCertificateAPI& api)
+public:
+	HAProxyHook(const std::shared_ptr<IOHookProvider>& Prov, StreamSocket* sock, UserCertificateAPI& api)
 		: IOHookMiddle(Prov)
-		, address_length(0)
 		, sslapi(api)
-		, state(HPS_WAITING_FOR_HEADER)
 	{
 		sock->AddIOHook(this);
 	}
 
-	bool IsHookReady() const CXX11_OVERRIDE
+	bool IsHookReady() const override
 	{
 		return state == HPS_CONNECTED;
 	}
 
-	int OnStreamSocketWrite(StreamSocket* sock, StreamSocket::SendQueue& uppersendq) CXX11_OVERRIDE
+	ssize_t OnStreamSocketWrite(StreamSocket* sock, StreamSocket::SendQueue& uppersendq) override
 	{
 		// We don't need to implement this.
 		GetSendQ().moveall(uppersendq);
 		return 1;
 	}
 
-	int OnStreamSocketRead(StreamSocket* sock, std::string& destrecvq) CXX11_OVERRIDE
+	ssize_t OnStreamSocketRead(StreamSocket* sock, std::string& destrecvq) override
 	{
 		switch (state)
 		{
@@ -413,33 +413,25 @@ class HAProxyHook : public IOHookMiddle
 		// We should never reach this point.
 		return -1;
 	}
-
-	void OnStreamSocketClose(StreamSocket* sock) CXX11_OVERRIDE
-	{
-		// We don't need to implement this.
-	}
 };
 
-void HAProxyHookProvider::OnAccept(StreamSocket* sock, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server)
+void HAProxyHookProvider::OnAccept(StreamSocket* sock, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server)
 {
-	new HAProxyHook(this, sock, sslapi);
+	new HAProxyHook(shared_from_this(), sock, sslapi);
 }
 
-class ModuleHAProxy : public Module
+class ModuleHAProxy final
+	: public Module
 {
- private:
-	reference<HAProxyHookProvider> hookprov;
+private:
+	std::shared_ptr<HAProxyHookProvider> hookprov;
 
- public:
+public:
 	ModuleHAProxy()
-		: hookprov(new HAProxyHookProvider(this))
+		: Module(VF_VENDOR, "Allows IRC connections to be made using reverse proxies that implement version 2 of the HAProxy PROXY protocol.")
+		, hookprov(std::make_shared<HAProxyHookProvider>(this))
 	{
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Allows IRC connections to be made using reverse proxies that implement version 2 of the HAProxy PROXY protocol.", VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleHAProxy)
