@@ -1,9 +1,8 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019 Robby <robby@chatbelgie.be>
- *   Copyright (C) 2013, 2017-2018, 2020-2022 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012-2014, 2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2019-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013-2014, 2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2010 Craig Edwards <brain@inspircd.org>
  *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
  *
@@ -24,11 +23,10 @@
 
 #define NETWORK_VALUE 9000000
 
-/** Handle /OJOIN
- */
-class CommandOjoin : public SplitCommand
+class CommandOjoin final
+	: public SplitCommand
 {
- public:
+public:
 	bool active;
 	bool notice;
 	bool op;
@@ -39,86 +37,90 @@ class CommandOjoin : public SplitCommand
 		, npmh(&mode)
 		, opmode(parent, "op")
 	{
-		flags_needed = 'o';
-		syntax = "<channel>";
+		access_needed = CmdAccess::OPERATOR;
+		syntax = { "<channel>" };
 		active = false;
 	}
 
-	CmdResult HandleLocal(LocalUser* user, const Params& parameters) CXX11_OVERRIDE
+	CmdResult HandleLocal(LocalUser* user, const Params& parameters) override
 	{
 		// Make sure the channel name is allowable.
-		if (!ServerInstance->IsChannel(parameters[0]))
+		if (!ServerInstance->Channels.IsChannel(parameters[0]))
 		{
 			user->WriteNumeric(ERR_BADCHANMASK, parameters[0], "Invalid channel name");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		active = true;
 		// override is false because we want OnUserPreJoin to run
-		Channel* channel = Channel::JoinUser(user, parameters[0], false);
+		Membership* memb = Channel::JoinUser(user, parameters[0], false);
 		active = false;
 
-		if (channel)
+		if (memb)
 		{
-			ServerInstance->SNO->WriteGlobalSno('a', user->nick+" used OJOIN to join "+channel->name);
+			ServerInstance->SNO.WriteGlobalSno('a', user->nick+" used OJOIN to join "+memb->chan->name);
 
 			if (notice)
-				channel->WriteRemoteNotice(user->nick + " joined on official network business.");
+				memb->chan->WriteRemoteNotice(user->nick + " joined on official network business.");
 		}
 		else
 		{
-			channel = ServerInstance->FindChan(parameters[0]);
+			Channel* channel = ServerInstance->Channels.Find(parameters[0]);
 			if (!channel)
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 
-			ServerInstance->SNO->WriteGlobalSno('a', user->nick+" used OJOIN in "+parameters[0]);
+			ServerInstance->SNO.WriteGlobalSno('a', user->nick+" used OJOIN in "+parameters[0]);
 			// they're already in the channel
 			Modes::ChangeList changelist;
 			changelist.push_add(npmh, user->nick);
 			if (op && opmode)
 				changelist.push_add(*opmode, user->nick);
-			ServerInstance->Modes->Process(ServerInstance->FakeClient, channel, NULL, changelist);
+			ServerInstance->Modes.Process(ServerInstance->FakeClient, channel, nullptr, changelist);
 		}
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 	}
 };
 
 /** channel mode +Y
  */
-class NetworkPrefix : public PrefixMode
+class NetworkPrefix final
+	: public PrefixMode
 {
- public:
+public:
 	NetworkPrefix(Module* parent, char NPrefix)
 		: PrefixMode(parent, "official-join", 'Y', NETWORK_VALUE, NPrefix)
 	{
-		ranktoset = ranktounset = UINT_MAX;
+		ranktoset = ranktounset = std::numeric_limits<ModeHandler::Rank>::max();
 	}
 
-	ModResult AccessCheck(User* source, Channel* channel, std::string &parameter, bool adding) CXX11_OVERRIDE
+	ModResult AccessCheck(User* source, Channel* channel, Modes::Change& change) override
 	{
-		User* theuser = ServerInstance->FindNick(parameter);
+		auto* theuser = ServerInstance->Users.Find(change.param);
 		// remove own privs?
-		if (source == theuser && !adding)
+		if (source == theuser && !change.adding)
 			return MOD_RES_ALLOW;
 
 		return MOD_RES_PASSTHRU;
 	}
 };
 
-class ModuleOjoin : public Module
+class ModuleOjoin final
+	: public Module
 {
+private:
 	NetworkPrefix np;
 	CommandOjoin mycommand;
 
- public:
+public:
 
 	ModuleOjoin()
-		: np(this, ServerInstance->Config->ConfValue("ojoin")->getCharacter("prefix"))
+		: Module(VF_VENDOR, "Adds the /OJOIN command which allows server operators to join a channel and receive the server operator-only Y (official-join) channel prefix mode.")
+		, np(this, ServerInstance->Config->ConfValue("ojoin")->getCharacter("prefix", '\0', true))
 		, mycommand(this, np)
 	{
 	}
 
-	ModResult OnUserPreJoin(LocalUser* user, Channel* chan, const std::string& cname, std::string& privs, const std::string& keygiven) CXX11_OVERRIDE
+	ModResult OnUserPreJoin(LocalUser* user, Channel* chan, const std::string& cname, std::string& privs, const std::string& keygiven, bool override) override
 	{
 		if (mycommand.active)
 		{
@@ -131,14 +133,14 @@ class ModuleOjoin : public Module
 		return MOD_RES_PASSTHRU;
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* Conf = ServerInstance->Config->ConfValue("ojoin");
+		const auto& Conf = ServerInstance->Config->ConfValue("ojoin");
 		mycommand.notice = Conf->getBool("notice", true);
 		mycommand.op = Conf->getBool("op", true);
 	}
 
-	ModResult OnUserPreKick(User* source, Membership* memb, const std::string &reason) CXX11_OVERRIDE
+	ModResult OnUserPreKick(User* source, Membership* memb, const std::string& reason) override
 	{
 		// Don't do anything if they're not +Y
 		if (!memb->HasMode(&np))
@@ -152,14 +154,9 @@ class ModuleOjoin : public Module
 		return MOD_RES_DENY;
 	}
 
-	void Prioritize() CXX11_OVERRIDE
+	void Prioritize() override
 	{
-		ServerInstance->Modules->SetPriority(this, I_OnUserPreJoin, PRIORITY_FIRST);
-	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds the /OJOIN command which allows server operators to join a channel and receive the server operator-only Y (official-join) channel prefix mode.", VF_VENDOR);
+		ServerInstance->Modules.SetPriority(this, I_OnUserPreJoin, PRIORITY_FIRST);
 	}
 };
 
