@@ -31,8 +31,12 @@
 #include "timeutils.h"
 #include "xline.h"
 
-static bool ZlineOnMatch = false;
-static bool added_zline = false;
+namespace
+{
+	bool addedzline = false;
+	bool useflags = false;
+	bool zlineonmatch = false;
+}
 
 class RLine final
 	: public XLine
@@ -45,7 +49,7 @@ public:
 		/* This can throw on failure, but if it does we DONT catch it here, we catch it and display it
 		 * where the object is created, we might not ALWAYS want it to output stuff to snomask x all the time
 		 */
-		regex = rxfactory->Create(regexs);
+		regex = useflags ? rxfactory->CreateHuman(regexs) : rxfactory->Create(regexs);
 	}
 
 	bool Matches(User* u) const override
@@ -66,7 +70,7 @@ public:
 
 	void Apply(User* u) override
 	{
-		if (ZlineOnMatch)
+		if (zlineonmatch)
 		{
 			auto* zl = new ZLine(ServerInstance->Time(), duration ? expiry - ServerInstance->Time() : 0, MODNAME "@" + ServerInstance->Config->ServerName, reason, u->GetAddress());
 			if (ServerInstance->XLines->AddLine(zl, nullptr))
@@ -79,10 +83,10 @@ public:
 				else
 				{
 					ServerInstance->SNO.WriteToSnoMask('x', "{} added a timed Z-line on {}, expires in {} (on {}): {}",
-						zl->source, u->GetAddress(), Duration::ToString(zl->duration),
+						zl->source, u->GetAddress(), Duration::ToLongString(zl->duration),
 						Time::ToString(zl->duration), zl->reason);
 				}
-				added_zline = true;
+				addedzline = true;
 			}
 			else
 				delete zl;
@@ -182,7 +186,7 @@ public:
 					else
 					{
 						ServerInstance->SNO.WriteToSnoMask('x', "{} added a timed R-line on {}, expires in {} (on {}): {}",
-							user->nick, parameters[0], Duration::ToString(duration), Time::FromNow(duration), r->reason);
+							user->nick, parameters[0], Duration::ToLongString(duration), Time::FromNow(duration), r->reason);
 					}
 
 					ServerInstance->XLines->ApplyLines();
@@ -220,13 +224,13 @@ private:
 	Regex::EngineReference rxfactory;
 	RLineFactory f;
 	CommandRLine r;
-	bool MatchOnNickChange;
+	bool matchonnickchange;
 	bool initing = true;
 	Regex::Engine* factory;
 
 public:
 	ModuleRLine()
-		: Module(VF_VENDOR | VF_COMMON, "Adds the /RLINE command which allows server operators to prevent users matching a nickname!username@hostname+realname regular expression from connecting to the server.")
+		: Module(VF_VENDOR | VF_COMMON, "Adds the /RLINE command which allows server operators to prevent users matching a \"nickname!username@hostname realname\" regular expression from connecting to the server.")
 		, Stats::EventListener(this)
 		, rxfactory(this)
 		, f(this, rxfactory)
@@ -254,6 +258,9 @@ public:
 		}
 		else
 			data["regex"] = "broken";
+
+		if (useflags)
+			data["flags"];
 	}
 
 	ModResult OnUserRegister(LocalUser* user) override
@@ -274,10 +281,11 @@ public:
 	{
 		const auto& tag = ServerInstance->Config->ConfValue("rline");
 
-		MatchOnNickChange = tag->getBool("matchonnickchange");
-		ZlineOnMatch = tag->getBool("zlineonmatch");
-		std::string newrxengine = tag->getString("engine");
+		matchonnickchange = tag->getBool("matchonnickchange");
+		useflags = tag->getBool("useflags");
+		zlineonmatch = tag->getBool("zlineonmatch");
 
+		std::string newrxengine = tag->getString("engine");
 		factory = rxfactory ? (rxfactory.operator->()) : nullptr;
 
 		rxfactory.SetEngine(newrxengine);
@@ -313,7 +321,7 @@ public:
 		if (!IS_LOCAL(user))
 			return;
 
-		if (!MatchOnNickChange)
+		if (!matchonnickchange)
 			return;
 
 		XLine* rl = ServerInstance->XLines->MatchesLine("R", user);
@@ -327,9 +335,9 @@ public:
 
 	void OnBackgroundTimer(time_t curtime) override
 	{
-		if (added_zline)
+		if (addedzline)
 		{
-			added_zline = false;
+			addedzline = false;
 			ServerInstance->XLines->ApplyLines();
 		}
 	}
