@@ -1,10 +1,16 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2019, 2021-2022 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2019 linuxdaemon <linuxdaemon.irc@gmail.com>
  *   Copyright (C) 2019 iwalkalone <iwalkalone69@gmail.com>
- *   Copyright (C) 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2018-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2015 Renegade334 <contact.caaeed4f@renegade334.me.uk>
+ *   Copyright (C) 2012, 2019 Robby <robby@chatbelgie.be>
+ *   Copyright (C) 2012 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2009-2010 Daniel De Graaf <danieldg@inspircd.org>
+ *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
+ *   Copyright (C) 2006-2007 Dennis Friis <peavey@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -21,7 +27,9 @@
 
 
 #include "inspircd.h"
+#include "clientprotocolmsg.h"
 #include "modules/ctctags.h"
+#include "modules/isupport.h"
 
 enum
 {
@@ -34,9 +42,9 @@ enum
 	ERR_SILENCE = 952
 };
 
-class SilenceEntry
+class SilenceEntry final
 {
- public:
+public:
 	enum SilenceFlags
 	{
 		// Does nothing; for internal use only.
@@ -45,7 +53,10 @@ class SilenceEntry
 		// Exclude users who match this flags ("x").
 		SF_EXEMPT = 1,
 
-		// 2, 4, 8, 16 are reserved for future use.
+		// Hide the contents of the message when sent to the silencer ("H").
+		SF_HIDE_SILENCER = 2,
+
+		// 4, 8, 16 are reserved for future use.
 
 		// Matches a NOTICE targeted at a channel ("n").
 		SF_NOTICE_CHANNEL = 32,
@@ -108,9 +119,9 @@ class SilenceEntry
 	static bool FlagsToBits(const std::string& flags, uint32_t& out, bool strict)
 	{
 		out = SF_NONE;
-		for (std::string::const_iterator flag = flags.begin(); flag != flags.end(); ++flag)
+		for (const auto flag : flags)
 		{
-			switch (*flag)
+			switch (flag)
 			{
 				case 'C':
 					out |= SF_CTCP_USER;
@@ -121,6 +132,9 @@ class SilenceEntry
 				case 'd':
 					out |= SF_DEFAULT;
 					break;
+				case 'H':
+					out |= SF_HIDE_SILENCER;
+					break;
 				case 'i':
 					out |= SF_INVITE;
 					break;
@@ -163,6 +177,8 @@ class SilenceEntry
 			out.push_back('C');
 		if (flags & SF_CTCP_CHANNEL)
 			out.push_back('c');
+		if (flags & SF_HIDE_SILENCER)
+			out.push_back('H');
 		if (flags & SF_INVITE)
 			out.push_back('i');
 		if (flags & SF_NOTICE_USER)
@@ -185,25 +201,29 @@ class SilenceEntry
 
 typedef insp::flat_set<SilenceEntry> SilenceList;
 
-class SilenceExtItem : public SimpleExtItem<SilenceList>
+class SilenceExtItem final
+	: public SimpleExtItem<SilenceList>
 {
- public:
-	unsigned int maxsilence;
+public:
+	unsigned long maxsilence;
 
 	SilenceExtItem(Module* Creator)
-		: SimpleExtItem<SilenceList>("silence_list", ExtensionItem::EXT_USER, Creator)
+		: SimpleExtItem<SilenceList>(Creator, "silence-list", ExtensionType::USER)
 	{
 	}
 
-	void FromInternal(Extensible* container, const std::string& value) CXX11_OVERRIDE
+	void FromInternal(Extensible* container, const std::string& value) noexcept override
 	{
+		if (container->extype != this->extype)
+			return;
+
 		LocalUser* user = IS_LOCAL(static_cast<User*>(container));
 		if (!user)
 			return;
 
 		// Remove the old list and create a new one.
-		unset(user);
-		SilenceList* list = new SilenceList();
+		Unset(user, false);
+		auto* list = new SilenceList();
 
 		irc::spacesepstream ts(value);
 		while (!ts.StreamEnd())
@@ -211,8 +231,8 @@ class SilenceExtItem : public SimpleExtItem<SilenceList>
 			// Check we have space for another entry.
 			if (list->size() >= maxsilence)
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Oversized silence list received for %s: %s",
-					user->uuid.c_str(), value.c_str());
+				ServerInstance->Logs.Debug(MODNAME, "Oversized silence list received for {}: {}",
+					user->uuid, value);
 				delete list;
 				return;
 			}
@@ -222,8 +242,8 @@ class SilenceExtItem : public SimpleExtItem<SilenceList>
 			std::string flagstr;
 			if (!ts.GetToken(mask) || !ts.GetToken(flagstr))
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Malformed silence list received for %s: %s",
-					user->uuid.c_str(), value.c_str());
+				ServerInstance->Logs.Debug(MODNAME, "Malformed silence list received for {}: {}",
+					user->uuid, value);
 				delete list;
 				return;
 			}
@@ -232,21 +252,28 @@ class SilenceExtItem : public SimpleExtItem<SilenceList>
 			uint32_t flags;
 			if (!SilenceEntry::FlagsToBits(flagstr, flags, false))
 			{
-				ServerInstance->Logs->Log(MODNAME, LOG_DEBUG, "Malformed silence flags received for %s: %s",
-					user->uuid.c_str(), flagstr.c_str());
+				ServerInstance->Logs.Debug(MODNAME, "Malformed silence flags received for {}: {}",
+					user->uuid, flagstr);
 				delete list;
 				return;
 			}
 
 			// Store the silence entry.
-			list->insert(SilenceEntry(flags, mask));
+			list->emplace(flags, mask);
+		}
+
+		// If we have an empty list then don't store it.
+		if (list->empty())
+		{
+			delete list;
+			return;
 		}
 
 		// The value was well formed.
-		set(user, list);
+		Set(user, list, false);
 	}
 
-	std::string ToInternal(const Extensible* container, void* item) const CXX11_OVERRIDE
+	std::string ToInternal(const Extensible* container, void* item) const noexcept override
 	{
 		SilenceList* list = static_cast<SilenceList*>(item);
 		std::string buf;
@@ -263,9 +290,10 @@ class SilenceExtItem : public SimpleExtItem<SilenceList>
 	}
 };
 
-class SilenceMessage : public ClientProtocol::Message
+class SilenceMessage final
+	: public ClientProtocol::Message
 {
- public:
+public:
 	SilenceMessage(const std::string& mask, const std::string& flags)
 		: ClientProtocol::Message("SILENCE")
 	{
@@ -274,40 +302,41 @@ class SilenceMessage : public ClientProtocol::Message
 	}
 };
 
-class CommandSilence : public SplitCommand
+class CommandSilence final
+	: public SplitCommand
 {
- private:
+private:
 	ClientProtocol::EventProvider msgprov;
 
 	CmdResult AddSilence(LocalUser* user, const std::string& mask, uint32_t flags)
 	{
-		SilenceList* list = ext.get(user);
+		SilenceList* list = ext.Get(user);
 		if (list && list->size() > ext.maxsilence)
 		{
 			user->WriteNumeric(ERR_SILELISTFULL, mask, SilenceEntry::BitsToFlags(flags), "Your SILENCE list is full");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 		else if (!list)
 		{
 			// There is no list; create it.
 			list = new SilenceList();
-			ext.set(user, list);
+			ext.Set(user, list);
 		}
 
-		if (!list->insert(SilenceEntry(flags, mask)).second)
+		if (!list->emplace(flags, mask).second)
 		{
 			user->WriteNumeric(ERR_SILENCE, mask, SilenceEntry::BitsToFlags(flags), "The SILENCE entry you specified already exists");
-			return CMD_FAILURE;
+			return CmdResult::FAILURE;
 		}
 
 		SilenceMessage msg("+" + mask, SilenceEntry::BitsToFlags(flags));
 		user->Send(msgprov, msg);
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 	}
 
 	CmdResult RemoveSilence(LocalUser* user, const std::string& mask, uint32_t flags)
 	{
-		SilenceList* list = ext.get(user);
+		SilenceList* list = ext.Get(user);
 		if (list)
 		{
 			for (SilenceList::iterator iter = list->begin(); iter != list->end(); ++iter)
@@ -318,29 +347,27 @@ class CommandSilence : public SplitCommand
 				list->erase(iter);
 				SilenceMessage msg("-" + mask, SilenceEntry::BitsToFlags(flags));
 				user->Send(msgprov, msg);
-				return CMD_SUCCESS;
+				return CmdResult::SUCCESS;
 			}
 		}
 
 		user->WriteNumeric(ERR_SILENCE, mask, SilenceEntry::BitsToFlags(flags), "The SILENCE entry you specified could not be found");
-		return CMD_FAILURE;
+		return CmdResult::FAILURE;
 	}
 
 	CmdResult ShowSilenceList(LocalUser* user)
 	{
-		SilenceList* list = ext.get(user);
+		SilenceList* list = ext.Get(user);
 		if (list)
 		{
-			for (SilenceList::const_iterator iter = list->begin(); iter != list->end(); ++iter)
-			{
-				user->WriteNumeric(RPL_SILELIST, iter->mask, SilenceEntry::BitsToFlags(iter->flags));
-			}
+			for (const auto& entry : *list)
+				user->WriteNumeric(RPL_SILELIST, entry.mask, SilenceEntry::BitsToFlags(entry.flags));
 		}
 		user->WriteNumeric(RPL_ENDOFSILELIST, "End of SILENCE list");
-		return CMD_SUCCESS;
+		return CmdResult::SUCCESS;
 	}
 
- public:
+public:
 	SilenceExtItem ext;
 
 	CommandSilence(Module* Creator)
@@ -348,11 +375,10 @@ class CommandSilence : public SplitCommand
 		, msgprov(Creator, "SILENCE")
 		, ext(Creator)
 	{
-		allow_empty_last_param = false;
-		syntax = "[(+|-)<mask> [CcdiNnPpTtx]]";
+		syntax = { "[(+|-)<mask> [CcdiNnPpTtx]]" };
 	}
 
-	CmdResult HandleLocal(LocalUser* user, const Params& parameters) CXX11_OVERRIDE
+	CmdResult HandleLocal(LocalUser* user, const Params& parameters) override
 	{
 		if (parameters.empty())
 			return ShowSilenceList(user);
@@ -378,7 +404,7 @@ class CommandSilence : public SplitCommand
 			if (!SilenceEntry::FlagsToBits(parameters[1], flags, true))
 			{
 				user->WriteNumeric(ERR_SILENCE, mask, parameters[1], "You specified one or more invalid SILENCE flags");
-				return CMD_FAILURE;
+				return CmdResult::FAILURE;
 			}
 			else if (flags == SilenceEntry::SF_EXEMPT)
 			{
@@ -391,78 +417,95 @@ class CommandSilence : public SplitCommand
 	}
 };
 
-class ModuleSilence
+class ModuleSilence final
 	: public Module
 	, public CTCTags::EventListener
+	, public ISupport::EventListener
 {
- private:
-	bool exemptuline;
+private:
+	bool exemptservice;
 	CommandSilence cmd;
 
-	ModResult BuildChannelExempts(User* source, Channel* channel, SilenceEntry::SilenceFlags flag, CUList& exemptions)
+	void BuildChannelExempts(User* source, Channel* channel, SilenceEntry::SilenceFlags flag, CUList& exemptions, CUList& hides)
 	{
-		const Channel::MemberMap& members = channel->GetUsers();
-		for (Channel::MemberMap::const_iterator member = members.begin(); member != members.end(); ++member)
+		for (const auto& [user, _] : channel->GetUsers())
 		{
-			if (!CanReceiveMessage(source, member->first, flag))
-				exemptions.insert(member->first);
+			uint32_t flags;
+			if (!CanReceiveMessage(source, user, flag, &flags))
+			{
+				exemptions.insert(user);
+				if (user != source && flags & SilenceEntry::SF_HIDE_SILENCER)
+					hides.insert(user);
+			}
 		}
-		return MOD_RES_PASSTHRU;
 	}
 
-	bool CanReceiveMessage(User* source, User* target, SilenceEntry::SilenceFlags flag)
+	bool CanReceiveMessage(User* source, User* target, SilenceEntry::SilenceFlags flag, uint32_t* flags = nullptr)
 	{
 		// Servers handle their own clients.
 		if (!IS_LOCAL(target))
 			return true;
 
-		if (exemptuline && source->server->IsULine())
+		if (exemptservice && source->server->IsService())
 			return true;
 
-		SilenceList* list = cmd.ext.get(target);
+		SilenceList* list = cmd.ext.Get(target);
 		if (!list)
 			return true;
 
-		for (SilenceList::iterator iter = list->begin(); iter != list->end(); ++iter)
+		for (const auto& entry : *list)
 		{
-			if (!(iter->flags & flag))
+			if (!(entry.flags & flag))
 				continue;
 
-			if (InspIRCd::Match(source->GetFullHost(), iter->mask))
-				return iter->flags & SilenceEntry::SF_EXEMPT;
+			if (InspIRCd::Match(source->GetMask(), entry.mask))
+			{
+				if (flags)
+					*flags = entry.flags;
+
+				return entry.flags & SilenceEntry::SF_EXEMPT;
+			}
 		}
 
 		return true;
 	}
 
- public:
+	void HideMessage(std::string& message)
+	{
+		InspIRCd::StripColor(message);
+		message.insert(0, "\x1DSilenced\x1D: \00315,15");
+	}
+
+public:
 	ModuleSilence()
-		: CTCTags::EventListener(this)
+		: Module(VF_VENDOR | VF_OPTCOMMON, "Adds the /SILENCE command which allows users to ignore other users on server-side.")
+		, CTCTags::EventListener(this)
+		, ISupport::EventListener(this)
 		, cmd(this)
 	{
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTag* tag = ServerInstance->Config->ConfValue("silence");
-		exemptuline = tag->getBool("exemptuline", true);
-		cmd.ext.maxsilence = tag->getUInt("maxentries", 32, 1);
+		const auto& tag = ServerInstance->Config->ConfValue("silence");
+		exemptservice = tag->getBool("exemptservice", tag->getBool("exemptuline", true));
+		cmd.ext.maxsilence = tag->getNum<unsigned long>("maxentries", 32, 1);
 	}
 
-	void On005Numeric(std::map<std::string, std::string>& tokens) CXX11_OVERRIDE
+	void OnBuildISupport(ISupport::TokenMap& tokens) override
 	{
 		tokens["ESILENCE"] = "CcdiNnPpTtx";
 		tokens["SILENCE"] = ConvToStr(cmd.ext.maxsilence);
 	}
 
-	ModResult OnUserPreInvite(User* source, User* dest, Channel* channel, time_t timeout) CXX11_OVERRIDE
+	ModResult OnUserPreInvite(User* source, User* dest, Channel* channel, time_t timeout) override
 	{
 		return CanReceiveMessage(source, dest, SilenceEntry::SF_INVITE) ? MOD_RES_PASSTHRU : MOD_RES_DENY;
 	}
 
-	ModResult OnUserPreMessage(User* user, const MessageTarget& target, MessageDetails& details) CXX11_OVERRIDE
+	ModResult OnUserPreMessage(User* user, MessageTarget& target, MessageDetails& details) override
 	{
-		std::string ctcpname;
+		std::string_view ctcpname;
 		bool is_ctcp = details.IsCTCP(ctcpname) && !irc::equals(ctcpname, "ACTION");
 
 		SilenceEntry::SilenceFlags flag = SilenceEntry::SF_NONE;
@@ -472,25 +515,45 @@ class ModuleSilence
 			{
 				if (is_ctcp)
 					flag = SilenceEntry::SF_CTCP_CHANNEL;
-				else if (details.type == MSG_NOTICE)
+				else if (details.type == MessageType::NOTICE)
 					flag = SilenceEntry::SF_NOTICE_CHANNEL;
-				else if (details.type == MSG_PRIVMSG)
+				else if (details.type == MessageType::PRIVMSG)
 					flag = SilenceEntry::SF_PRIVMSG_CHANNEL;
 
-				return BuildChannelExempts(user, target.Get<Channel>(), flag, details.exemptions);
+				CUList hides;
+				BuildChannelExempts(user, target.Get<Channel>(), flag, details.exemptions, hides);
+				if (!hides.empty())
+				{
+					// In this mode we just strip formatting and hide the message.
+					std::string message = details.text;
+					HideMessage(message);
+
+					// Servers handle their own users so this cast will always work.
+					ClientProtocol::Messages::Privmsg msg(user, target.Get<Channel>(), message, details.type, target.status);
+					for (auto* hideuser : hides)
+						static_cast<LocalUser*>(hideuser)->Send(ServerInstance->GetRFCEvents().privmsg, msg);
+				}
+				return MOD_RES_PASSTHRU;
 			}
 			case MessageTarget::TYPE_USER:
 			{
 				if (is_ctcp)
 					flag = SilenceEntry::SF_CTCP_USER;
-				else if (details.type == MSG_NOTICE)
+				else if (details.type == MessageType::NOTICE)
 					flag = SilenceEntry::SF_NOTICE_USER;
-				else if (details.type == MSG_PRIVMSG)
+				else if (details.type == MessageType::PRIVMSG)
 					flag = SilenceEntry::SF_PRIVMSG_USER;
 
-				if (!CanReceiveMessage(user, target.Get<User>(), flag))
+				uint32_t flags;
+				if (!CanReceiveMessage(user, target.Get<User>(), flag, &flags))
 				{
 					details.echo_original = true;
+					if (flags & SilenceEntry::SF_HIDE_SILENCER)
+					{
+						// In this mode we just strip formatting and hide the message.
+						HideMessage(details.text);
+						break;
+					}
 					return MOD_RES_DENY;
 				}
 				break;
@@ -502,10 +565,14 @@ class ModuleSilence
 		return MOD_RES_PASSTHRU;
 	}
 
-	ModResult OnUserPreTagMessage(User* user, const MessageTarget& target, CTCTags::TagMessageDetails& details) CXX11_OVERRIDE
+	ModResult OnUserPreTagMessage(User* user, MessageTarget& target, CTCTags::TagMessageDetails& details) override
 	{
 		if (target.type == MessageTarget::TYPE_CHANNEL)
-			return BuildChannelExempts(user, target.Get<Channel>(), SilenceEntry::SF_TAGMSG_CHANNEL, details.exemptions);
+		{
+			CUList unused;
+			BuildChannelExempts(user, target.Get<Channel>(), SilenceEntry::SF_TAGMSG_CHANNEL, details.exemptions, unused);
+			return MOD_RES_PASSTHRU;
+		}
 
 		if (target.type == MessageTarget::TYPE_USER && !CanReceiveMessage(user, target.Get<User>(), SilenceEntry::SF_TAGMSG_USER))
 		{
@@ -515,11 +582,6 @@ class ModuleSilence
 
 		return MOD_RES_PASSTHRU;
 	}
-
-	Version GetVersion() CXX11_OVERRIDE
-	{
-		return Version("Adds the /SILENCE command which allows users to ignore other users on server-side.", VF_OPTCOMMON | VF_VENDOR);
-	}
 };
 
 MODULE_INIT(ModuleSilence)
