@@ -1,7 +1,7 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013-2014, 2018-2024 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2013-2014, 2018-2025 Sadie Powell <sadie@witchery.services>
  *   Copyright (C) 2013 Adam <Adam@anope.org>
  *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
@@ -294,6 +294,12 @@ void SpanningTreeUtilities::ReadConfiguration()
 		L->Bind = tag->getString("bind");
 		L->Hidden = tag->getBool("hidden");
 
+		if (!tag->getString("ssl").empty())
+		{
+			throw ModuleException((Module*)Creator, "Obsolete TLS configuration in link block at {}. See {}modules/spanningtree/#link for the correct way to configure TLS.",
+				tag->source.str(), INSPIRCD_DOCS);
+		}
+
 		if (L->Name.empty())
 			throw ModuleException((Module*)Creator, "Invalid configuration, found a link tag without a name!" + (!L->IPAddr.empty() ? " IP address: "+L->IPAddr : ""));
 
@@ -301,7 +307,7 @@ void SpanningTreeUtilities::ReadConfiguration()
 			throw ModuleException((Module*)Creator, "The link name '"+L->Name+"' is invalid as it must contain at least one '.' character");
 
 		if (L->Name.length() > ServerInstance->Config->Limits.MaxHost)
-			throw ModuleException((Module*)Creator, "The link name '"+L->Name+"' is invalid as it is longer than " + ConvToStr(ServerInstance->Config->Limits.MaxHost) + " characters");
+			throw ModuleException((Module*)Creator, "The link name '{}' is invalid as it is longer than {} characters", L->Name, ServerInstance->Config->Limits.MaxHost);
 
 		if (L->RecvPass.empty())
 			throw ModuleException((Module*)Creator, "Invalid configuration for server '"+L->Name+"', recvpass not defined");
@@ -405,7 +411,9 @@ std::string SpanningTreeUtilities::BuildLinkString(uint16_t proto, Module* mod)
 		if (iter != data.begin())
 			buffer << '&';
 
-		buffer << iter->first << '=' << Percent::Encode(iter->second);
+		buffer << iter->first;
+		if (!iter->second.empty())
+			buffer << '=' << Percent::Encode(iter->second);
 	}
 	return buffer.str();
 }
