@@ -1,13 +1,14 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
- *   Copyright (C) 2013, 2017-2019, 2021 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2012-2016 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2021 Dominic Hamon
+ *   Copyright (C) 2013, 2017-2018, 2020-2023 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2012-2013, 2015-2016 Attila Molnar <attilamolnar@hush.com>
  *   Copyright (C) 2012 Robby <robby@chatbelgie.be>
  *   Copyright (C) 2009 Daniel De Graaf <danieldg@inspircd.org>
  *   Copyright (C) 2008 Robin Burchell <robin+git@viroteck.net>
  *   Copyright (C) 2007-2008 Dennis Friis <peavey@inspircd.org>
- *   Copyright (C) 2007-2008 Craig Edwards <brain@inspircd.org>
+ *   Copyright (C) 2007 Craig Edwards <brain@inspircd.org>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -40,18 +41,19 @@ extern SpanningTreeUtilities* Utils;
 
 /** Associative container type, mapping server names/ids to TreeServers
  */
-typedef TR1NS::unordered_map<std::string, TreeServer*, irc::insensitive, irc::StrHashComp> server_hash;
+typedef std::unordered_map<std::string, TreeServer*, irc::insensitive, irc::StrHashComp> server_hash;
 
 /** Contains helper functions and variables for this module,
  * and keeps them out of the global namespace
  */
-class SpanningTreeUtilities : public classbase
+class SpanningTreeUtilities final
+	: public Cullable
 {
 	CacheRefreshTimer RefreshTimer;
 
- public:
+public:
 	typedef std::set<TreeSocket*> TreeSocketSet;
-	typedef std::map<TreeSocket*, std::pair<std::string, unsigned int> > TimeoutList;
+	typedef std::map<TreeSocket*, std::pair<std::string, unsigned int>> TimeoutList;
 
 	/** Creator module
 	 */
@@ -67,7 +69,7 @@ class SpanningTreeUtilities : public classbase
 
 	/** Hide U-Lined servers in /MAP and /LINKS
 	 */
-	bool HideULines;
+	bool HideServices;
 	/** Announce TS changes to channels on merge
 	 */
 	bool AnnounceTSChange;
@@ -83,10 +85,10 @@ class SpanningTreeUtilities : public classbase
 	/* Number of seconds that a server can go without ping
 	 * before opers are warned of high latency.
 	 */
-	unsigned int PingWarnTime;
+	unsigned long PingWarnTime;
 	/** This variable represents the root of the server tree
 	 */
-	TreeServer *TreeRoot;
+	TreeServer* TreeRoot = nullptr;
 	/** IPs allowed to link to us
 	 */
 	std::vector<std::string> ValidIPs;
@@ -101,14 +103,14 @@ class SpanningTreeUtilities : public classbase
 	TimeoutList timeoutlist;
 	/** Holds the data from the <link> tags in the conf
 	 */
-	std::vector<reference<Link> > LinkBlocks;
+	std::vector<std::shared_ptr<Link>> LinkBlocks;
 	/** Holds the data from the <autoconnect> tags in the conf
 	 */
-	std::vector<reference<Autoconnect> > AutoconnectBlocks;
+	std::vector<std::shared_ptr<Autoconnect>> AutoconnectBlocks;
 
 	/** Ping frequency of server to server links
 	 */
-	unsigned int PingFreq;
+	unsigned long PingFreq = 60;
 
 	/** Initialise utility class
 	 */
@@ -116,25 +118,21 @@ class SpanningTreeUtilities : public classbase
 
 	/** Prepare for class destruction
 	 */
-	CullResult cull() CXX11_OVERRIDE;
+	Cullable::Result Cull() override;
 
 	/** Destroy class and free listeners etc
 	 */
-	~SpanningTreeUtilities();
+	~SpanningTreeUtilities() override;
 
 	void RouteCommand(TreeServer* origin, CommandBase* cmd, const CommandBase::Params& parameters, User* user);
 
 	/** Send a message from this server to one other local or remote
 	 */
-	void DoOneToOne(const CmdBuilder& params, Server* target);
+	static void DoOneToOne(const CmdBuilder& params, const Server* target);
 
 	/** Send a message from this server to all but one other, local or remote
 	 */
-	void DoOneToAllButSender(const CmdBuilder& params, TreeServer* omit);
-
-	/** Send a message from this server to all others
-	 */
-	void DoOneToMany(const CmdBuilder& params);
+	void DoOneToAllButSender(const CmdBuilder& params, const TreeServer* omit) const;
 
 	/** Read the spanningtree module's tags from the config file
 	 */
@@ -142,19 +140,19 @@ class SpanningTreeUtilities : public classbase
 
 	/** Handle nick collision
 	 */
-	bool DoCollision(User* u, TreeServer* server, time_t remotets, const std::string& remoteident, const std::string& remoteip, const std::string& remoteuid, const char* collidecmd);
+	static bool DoCollision(User* u, TreeServer* server, time_t remotets, const std::string& remoteuser, const std::string& remoteip, const std::string& remoteuid, const char* collidecmd);
 
 	/** Compile a list of servers which contain members of channel c
 	 */
-	void GetListOfServersForChannel(Channel* c, TreeSocketSet& list, char status, const CUList& exempt_list);
+	void GetListOfServersForChannel(const Channel* c, TreeSocketSet& list, char status, const CUList& exempt_list) const;
 
 	/** Find a server by name or SID
 	 */
-	TreeServer* FindServer(const std::string &ServerName);
+	TreeServer* FindServer(const std::string& ServerName);
 
 	/** Find server by SID
 	 */
-	TreeServer* FindServerID(const std::string &id);
+	TreeServer* FindServerID(const std::string& id);
 
 	/** Find a server based on a target string.
 	 * @param target Target string where a command should be routed to. May be a server name, a sid, a nickname or a uuid.
@@ -163,11 +161,11 @@ class SpanningTreeUtilities : public classbase
 
 	/** Find a server by glob mask
 	 */
-	TreeServer* FindServerMask(const std::string &ServerName);
+	TreeServer* FindServerMask(const std::string& ServerName);
 
 	/** Find a link tag from a server name
 	 */
-	Link* FindLink(const std::string& name);
+	std::shared_ptr<Link> FindLink(const std::string& name);
 
 	/** Refresh the IP cache used for allowing inbound connections
 	 */
@@ -175,13 +173,12 @@ class SpanningTreeUtilities : public classbase
 
 	/** Sends a PRIVMSG or a NOTICE to a channel obeying an exempt list and an optional prefix
 	 */
-	void SendChannelMessage(User* source, Channel* target, const std::string& text, char status, const ClientProtocol::TagMap& tags, const CUList& exempt_list, const char* message_type, TreeSocket* omit = NULL);
+	void SendChannelMessage(const User* source, const Channel* target, const std::string& text, char status, const ClientProtocol::TagMap& tags, const CUList& exempt_list, const char* message_type, const TreeSocket* omit = nullptr) const;
 
-	/** Send the channel list mode limits to either the specified server or all servers if NULL. */
-	void SendListLimits(Channel* chan, TreeSocket* sock = NULL);
+	// Builds link data to be sent to another server.
+	static std::string BuildLinkString(uint16_t protocol, Module* mod);
+
+	/** Send the channel list mode limits to either the specified server or all servers if nullptr. */
+	static void SendListLimits(Channel* chan, TreeSocket* sock = nullptr);
 };
 
-inline void SpanningTreeUtilities::DoOneToMany(const CmdBuilder& params)
-{
-	DoOneToAllButSender(params, NULL);
-}
