@@ -1,9 +1,10 @@
 /*
  * InspIRCd -- Internet Relay Chat Daemon
  *
+ *   Copyright (C) 2021 Dominic Hamon
  *   Copyright (C) 2019 iwalkalone <iwalkalone69@gmail.com>
- *   Copyright (C) 2017-2022 Sadie Powell <sadie@witchery.services>
- *   Copyright (C) 2016-2017 Attila Molnar <attilamolnar@hush.com>
+ *   Copyright (C) 2018-2025 Sadie Powell <sadie@witchery.services>
+ *   Copyright (C) 2016 Attila Molnar <attilamolnar@hush.com>
  *
  * This file is part of InspIRCd.  InspIRCd is free software: you can
  * redistribute it and/or modify it under the terms of the GNU General Public
@@ -18,22 +19,53 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/// $CompilerFlags: -Ivendor_directory("utfcpp")
+/// $CompilerFlags: require_environment("SYSTEM_UTFCPP" "1") -DUSE_SYSTEM_UTFCPP
 
 
+#ifdef USE_SYSTEM_UTFCPP
+# include <utf8cpp/utf8/unchecked.h>
+#else
+# include <utfcpp/unchecked.h>
+#endif
+
 #include "inspircd.h"
+#include "extension.h"
 #include "iohook.h"
 #include "modules/hash.h"
+#include "modules/isupport.h"
+#include "modules/whois.h"
+#include "utility/string.h"
 
-#define UTF_CPP_CPLUSPLUS 199711L
-#include <unchecked.h>
+static constexpr char MagicGUID[] = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11";
+static constexpr char newline[] = "\r\n";
+static constexpr char whitespace[] = " \t";
 
-static const char MagicGUID[] = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11";
-static const char newline[] = "\r\n";
-static const char whitespace[] = " \t";
-static dynamic_reference_nocheck<HashProvider>* sha1;
+struct SharedData final
+{
+	// An extension that stores the value of the Origin header.
+	StringExtItem origin;
 
-struct WebSocketConfig
+	// An extension that stores the real hostname for proxied hosts.
+	StringExtItem realhost;
+
+	// An extension that stores the real IP for proxied hosts.
+	StringExtItem realip;
+
+	// A reference to the SHA-1 provider.
+	dynamic_reference_nocheck<HashProvider> sha1;
+
+	SharedData(Module* mod)
+		: origin(mod, "websocket-origin", ExtensionType::USER, true)
+		, realhost(mod, "websocket-realhost", ExtensionType::USER, true)
+		, realip(mod, "websocket-realip", ExtensionType::USER, true)
+		, sha1(mod, "hash/sha1")
+	{
+	}
+};
+
+static SharedData* g_data;
+
+struct WebSocketConfig final
 {
 	enum DefaultMode
 	{
@@ -53,38 +85,49 @@ struct WebSocketConfig
 	// The HTTP origins that can connect to the server.
 	OriginList allowedorigins;
 
+	// Whether text encoding can be used on this server.
+	bool allowtext;
+
 	// The method to use if a subprotocol is not negotiated.
 	DefaultMode defaultmode;
 
 	// The IP ranges which send trustworthy X-Real-IP or X-Forwarded-For headers.
 	ProxyRanges proxyranges;
+
+	// Whether to allow connections from clients that do not send an Origin header.
+	bool allowmissingorigin;
+
+	// Whether to send WebSocket ping messages instead of IRC ping messages.
+	bool nativeping;
 };
 
-class WebSocketHookProvider : public IOHookProvider
+class WebSocketHookProvider final
+	: public IOHookProvider
 {
- public:
+public:
 	WebSocketConfig config;
 	WebSocketHookProvider(Module* mod)
 		: IOHookProvider(mod, "websocket", IOHookProvider::IOH_UNKNOWN, true)
 	{
 	}
 
-	void OnAccept(StreamSocket* sock, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server) CXX11_OVERRIDE;
+	void OnAccept(StreamSocket* sock, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server) override;
 
-	void OnConnect(StreamSocket* sock) CXX11_OVERRIDE
+	void OnConnect(StreamSocket* sock) override
 	{
 	}
 };
 
-class WebSocketHook : public IOHookMiddle
+class WebSocketHook final
+	: public IOHookMiddle
 {
-	class HTTPHeaderFinder
+	class HTTPHeaderFinder final
 	{
-	 private:
+	private:
 		std::string::size_type bpos;
 		std::string::size_type len;
 
-	 public:
+	public:
 		bool Find(const std::string& req, const char* header, std::string::size_type headerlen, std::string::size_type maxpos)
 		{
 			// Skip the GET /wibble HTTP/1.1 line.
@@ -143,19 +186,19 @@ class WebSocketHook : public IOHookMiddle
 		STATE_ESTABLISHED
 	};
 
-	static const unsigned char WS_MASKBIT = (1 << 7);
-	static const unsigned char WS_FINBIT = (1 << 7);
-	static const unsigned char WS_PAYLOAD_LENGTH_MAGIC_LARGE = 126;
-	static const unsigned char WS_PAYLOAD_LENGTH_MAGIC_HUGE = 127;
-	static const size_t WS_MAX_PAYLOAD_LENGTH_SMALL = 125;
-	static const size_t WS_MAX_PAYLOAD_LENGTH_LARGE = 65535;
-	static const size_t MAXHEADERSIZE = sizeof(uint64_t) + 2;
+	static constexpr unsigned char WS_MASKBIT = (1 << 7);
+	static constexpr unsigned char WS_FINBIT = (1 << 7);
+	static constexpr unsigned char WS_PAYLOAD_LENGTH_MAGIC_LARGE = 126;
+	static constexpr unsigned char WS_PAYLOAD_LENGTH_MAGIC_HUGE = 127;
+	static constexpr size_t WS_MAX_PAYLOAD_LENGTH_SMALL = 125;
+	static constexpr size_t WS_MAX_PAYLOAD_LENGTH_LARGE = 65535;
+	static constexpr size_t MAXHEADERSIZE = sizeof(uint64_t) + 2;
 
 	// Clients sending ping or pong frames faster than this are killed
-	static const time_t MINPINGPONGDELAY = 10;
+	static constexpr time_t MINPINGPONGDELAY = 10;
 
-	State state;
-	time_t lastpingpong;
+	State state = STATE_HTTPREQ;
+	time_t lastpingpong = 0;
 	WebSocketConfig& config;
 	bool sendastext;
 
@@ -276,15 +319,23 @@ class WebSocketHook : public IOHookMiddle
 
 		std::string appdata;
 		const int result = HandleAppData(sock, appdata, false);
-		// If it's a pong stop here regardless of the result so we won't generate a reply
-		if ((result <= 0) || (!isping))
+		if (result <= 0)
 			return result;
 
-		StreamSocket::SendQueue::Element elem = PrepareSendQElem(appdata.length(), OP_PONG);
-		elem.append(appdata);
-		GetSendQ().push_back(elem);
+		if (isping)
+		{
+			StreamSocket::SendQueue::Element elem = PrepareSendQElem(appdata.length(), OP_PONG);
+			elem.append(appdata);
+			GetSendQ().push_back(elem);
 
-		SocketEngine::ChangeEventMask(sock, FD_ADD_TRIAL_WRITE);
+			SocketEngine::ChangeEventMask(sock, FD_ADD_TRIAL_WRITE);
+		}
+		else if (sock->type == StreamSocket::SS_USER && config.nativeping)
+		{
+			// Pong reply on user socket; reset their idle time.
+			UserIOHandler* ioh = static_cast<UserIOHandler*>(sock);
+			ioh->user->lastping = 1;
+		}
 		return 1;
 	}
 
@@ -293,7 +344,7 @@ class WebSocketHook : public IOHookMiddle
 		if (GetRecvQ().empty())
 			return 0;
 
-		unsigned char opcode = (unsigned char)GetRecvQ().c_str()[0];
+		unsigned char opcode = (unsigned char)GetRecvQ()[0];
 		switch (opcode & ~WS_FINBIT)
 		{
 			case OP_CONTINUATION:
@@ -306,10 +357,10 @@ class WebSocketHook : public IOHookMiddle
 					return result;
 
 				// Strip out any CR+LF which may have been erroneously sent.
-				for (std::string::const_iterator iter = appdata.begin(); iter != appdata.end(); ++iter)
+				for (const auto chr : appdata)
 				{
-					if (*iter != '\r' && *iter != '\n')
-						destrecvq.push_back(*iter);
+					if (chr != '\r' && chr != '\n')
+						destrecvq.push_back(chr);
 				}
 
 				// If we are on the final message of this block append a line terminator.
@@ -374,20 +425,31 @@ class WebSocketHook : public IOHookMiddle
 		if (reqend == std::string::npos)
 			return 0;
 
+		LocalUser* luser = nullptr;
+		if (sock->type == StreamSocket::SS_USER)
+			luser = static_cast<UserIOHandler*>(sock)->user;
+
 		bool allowedorigin = false;
 		HTTPHeaderFinder originheader;
 		if (originheader.Find(recvq, "Origin:", 7, reqend))
 		{
 			const std::string origin = originheader.ExtractValue(recvq);
-			for (WebSocketConfig::OriginList::const_iterator iter = config.allowedorigins.begin(); iter != config.allowedorigins.end(); ++iter)
+			for (const auto& cfgorigin : config.allowedorigins)
 			{
-				if (InspIRCd::Match(origin, *iter, ascii_case_insensitive_map))
+				if (InspIRCd::Match(origin, cfgorigin, ascii_case_insensitive_map))
 				{
 					allowedorigin = true;
+					if (luser)
+						g_data->origin.Set(luser, origin);
 					break;
 				}
 			}
 		}
+		else if (config.allowmissingorigin)
+		{
+			// This is a non-web WebSocket connection.
+			allowedorigin = true;
+		}
 		else
 		{
 			FailHandshake(sock, "HTTP/1.1 400 Bad Request\r\nConnection: close\r\n\r\n", "WebSocket: Received HTTP request that did not send the Origin header");
@@ -400,16 +462,20 @@ class WebSocketHook : public IOHookMiddle
 			return -1;
 		}
 
-		if (!config.proxyranges.empty() && sock->type == StreamSocket::SS_USER)
+		if (!config.proxyranges.empty() && luser)
 		{
-			LocalUser* luser = static_cast<UserIOHandler*>(sock)->user;
 			irc::sockets::sockaddrs realsa(luser->client_sa);
 
 			HTTPHeaderFinder proxyheader;
 			if (proxyheader.Find(recvq, "X-Real-IP:", 10, reqend) || proxyheader.Find(recvq, "X-Forwarded-For:", 16, reqend))
 			{
 				// Attempt to parse the proxy HTTP header.
-				irc::sockets::aptosa(proxyheader.ExtractValue(recvq), realsa.port(), realsa);
+				if (!realsa.from_ip_port(proxyheader.ExtractValue(recvq), realsa.port()))
+				{
+					// The proxy header value contains a malformed value.
+					FailHandshake(sock, "HTTP/1.1 400 Bad Request\r\nConnection: close\r\n\r\n", "WebSocket: Received a proxied HTTP request that sent a malformed real IP address");
+					return -1;
+				}
 			}
 			else
 			{
@@ -417,21 +483,17 @@ class WebSocketHook : public IOHookMiddle
 				FailHandshake(sock, "HTTP/1.1 400 Bad Request\r\nConnection: close\r\n\r\n", "WebSocket: Received a proxied HTTP request that did not send a real IP address header");
 				return -1;
 			}
-
-			if (realsa.family() == AF_UNSPEC)
+			for (const auto& proxyrange : config.proxyranges)
 			{
-				// The proxy header value contains a malformed value.
-				FailHandshake(sock, "HTTP/1.1 400 Bad Request\r\nConnection: close\r\n\r\n", "WebSocket: Received a proxied HTTP request that sent a malformed real IP address");
-				return -1;
-			}
-
-			for (WebSocketConfig::ProxyRanges::const_iterator iter = config.proxyranges.begin(); iter != config.proxyranges.end(); ++iter)
-			{
-				if (InspIRCd::MatchCIDR(luser->GetIPString(), *iter, ascii_case_insensitive_map))
+				if (InspIRCd::MatchCIDR(luser->GetAddress(), proxyrange, ascii_case_insensitive_map))
 				{
 					// Give the user their real IP address.
 					if (realsa != luser->client_sa)
-						luser->SetClientIP(realsa);
+					{
+						g_data->realhost.Set(luser, luser->GetRealHost());
+						g_data->realip.Set(luser, luser->GetAddress());
+						luser->ChangeRemoteAddress(realsa);
+					}
 
 					// Error if changing their IP gets them banned.
 					if (luser->quitting)
@@ -450,15 +512,15 @@ class WebSocketHook : public IOHookMiddle
 			{
 				proto.erase(std::remove_if(proto.begin(), proto.end(), ::isspace), proto.end());
 
-				bool is_binary = (sock->type == StreamSocket::SS_USER && stdalgo::string::equalsci(proto, "binary.ircv3.net"))
-					|| stdalgo::string::equalsci(proto, "binary.inspircd.org");
+				auto is_binary = (sock->type == StreamSocket::SS_USER && insp::equalsci(proto, "binary.ircv3.net"))
+					|| insp::equalsci(proto, "binary.inspircd.org");
 
-				bool is_text = (sock->type == StreamSocket::SS_USER && stdalgo::string::equalsci(proto, "text.ircv3.net"))
-					|| stdalgo::string::equalsci(proto, "text.inspircd.org");
+				auto is_text = (sock->type == StreamSocket::SS_USER && insp::equalsci(proto, "text.ircv3.net"))
+					|| insp::equalsci(proto, "text.inspircd.org");
 
 				if (is_binary || is_text)
 				{
-					selectedproto = proto;
+					selectedproto = std::move(proto);
 					sendastext = is_text;
 					break;
 				}
@@ -471,6 +533,12 @@ class WebSocketHook : public IOHookMiddle
 			return -1;
 		}
 
+		if (sendastext && !config.allowtext)
+		{
+			FailHandshake(sock, "HTTP/1.1 400 Bad Request\r\nConnection: close\r\n\r\n", "WebSocket: Received HTTP request that requested a text protocol which is not compatible with the server character set");
+			return -1;
+		}
+
 		HTTPHeaderFinder keyheader;
 		if (!keyheader.Find(recvq, "Sec-WebSocket-Key:", 18, reqend))
 		{
@@ -478,7 +546,7 @@ class WebSocketHook : public IOHookMiddle
 			return -1;
 		}
 
-		if (!*sha1)
+		if (!*g_data->sha1)
 		{
 			FailHandshake(sock, "HTTP/1.1 503 Service Unavailable\r\nConnection: close\r\n\r\n", "WebSocket: SHA-1 provider missing");
 			return -1;
@@ -490,7 +558,7 @@ class WebSocketHook : public IOHookMiddle
 		key.append(MagicGUID);
 
 		std::string reply = "HTTP/1.1 101 Switching Protocols\r\nUpgrade: websocket\r\nConnection: Upgrade\r\nSec-WebSocket-Accept: ";
-		reply.append(BinToBase64((*sha1)->GenerateRaw(key), NULL, '=')).append(newline);
+		reply.append(Base64::Encode((*g_data->sha1)->GenerateRaw(key), nullptr, '=')).append(newline);
 		if (!selectedproto.empty())
 			reply.append("Sec-WebSocket-Protocol: ").append(selectedproto).append(newline);
 		reply.append(newline);
@@ -503,23 +571,21 @@ class WebSocketHook : public IOHookMiddle
 		return 1;
 	}
 
- public:
-	WebSocketHook(IOHookProvider* Prov, StreamSocket* sock, WebSocketConfig& cfg)
+public:
+	WebSocketHook(const std::shared_ptr<IOHookProvider>& Prov, StreamSocket* sock, WebSocketConfig& cfg)
 		: IOHookMiddle(Prov)
-		, state(STATE_HTTPREQ)
-		, lastpingpong(0)
 		, config(cfg)
 		, sendastext(config.defaultmode != WebSocketConfig::DM_BINARY)
 	{
 		sock->AddIOHook(this);
 	}
 
-	bool IsHookReady() const CXX11_OVERRIDE
+	bool IsHookReady() const override
 	{
 		return state == STATE_ESTABLISHED;
 	}
 
-	int OnStreamSocketWrite(StreamSocket* sock, StreamSocket::SendQueue& uppersendq) CXX11_OVERRIDE
+	ssize_t OnStreamSocketWrite(StreamSocket* sock, StreamSocket::SendQueue& uppersendq) override
 	{
 		StreamSocket::SendQueue& mysendq = GetSendQ();
 
@@ -528,11 +594,11 @@ class WebSocketHook : public IOHookMiddle
 			return (mysendq.empty() ? 0 : 1);
 
 		std::string message;
-		for (StreamSocket::SendQueue::const_iterator elem = uppersendq.begin(); elem != uppersendq.end(); ++elem)
+		for (const auto& elem : uppersendq)
 		{
-			for (StreamSocket::SendQueue::Element::const_iterator chr = elem->begin(); chr != elem->end(); ++chr)
+			for (const auto chr : elem)
 			{
-				if (*chr == '\n')
+				if (chr == '\n')
 				{
 					// We have found an entire message. Send it in its own frame.
 					if (sendastext)
@@ -552,9 +618,9 @@ class WebSocketHook : public IOHookMiddle
 					}
 					message.clear();
 				}
-				else if (*chr != '\r')
+				else if (chr != '\r')
 				{
-					message.push_back(*chr);
+					message.push_back(chr);
 				}
 			}
 		}
@@ -570,7 +636,7 @@ class WebSocketHook : public IOHookMiddle
 		return 1;
 	}
 
-	int OnStreamSocketRead(StreamSocket* sock, std::string& destrecvq) CXX11_OVERRIDE
+	ssize_t OnStreamSocketRead(StreamSocket* sock, std::string& destrecvq) override
 	{
 		if (state == STATE_HTTPREQ)
 		{
@@ -589,71 +655,102 @@ class WebSocketHook : public IOHookMiddle
 		return wsret;
 	}
 
-	void OnStreamSocketClose(StreamSocket* sock) CXX11_OVERRIDE
+	bool Ping() override
 	{
+		if (!config.nativeping)
+			return false;
+
+		StreamSocket::SendQueue& mysendq = GetSendQ();
+
+		const std::string& message = ServerInstance->Config->GetServerName();
+		mysendq.push_back(PrepareSendQElem(message.length(), OP_PING));
+		mysendq.push_back(message);
+
+		return true;
 	}
 };
 
-void WebSocketHookProvider::OnAccept(StreamSocket* sock, irc::sockets::sockaddrs* client, irc::sockets::sockaddrs* server)
+void WebSocketHookProvider::OnAccept(StreamSocket* sock, const irc::sockets::sockaddrs& client, const irc::sockets::sockaddrs& server)
 {
-	new WebSocketHook(this, sock, config);
+	new WebSocketHook(shared_from_this(), sock, config);
 }
 
-class ModuleWebSocket : public Module
+class ModuleWebSocket final
+	: public Module
+	, public Whois::EventListener
 {
-	dynamic_reference_nocheck<HashProvider> hash;
-	reference<WebSocketHookProvider> hookprov;
+private:
+	std::shared_ptr<WebSocketHookProvider> hookprov;
+	ISupport::EventProvider isupportprov;
+	SharedData data;
 
- public:
+public:
 	ModuleWebSocket()
-		: hash(this, "hash/sha1")
-		, hookprov(new WebSocketHookProvider(this))
+		: Module(VF_VENDOR, "Allows WebSocket clients to connect to the IRC server.")
+		, Whois::EventListener(this)
+		, hookprov(std::make_shared<WebSocketHookProvider>(this))
+		, isupportprov(this)
+		, data(this)
 	{
-		sha1 = &hash;
+		g_data = &data;
 	}
 
-	void ReadConfig(ConfigStatus& status) CXX11_OVERRIDE
+	void ReadConfig(ConfigStatus& status) override
 	{
-		ConfigTagList tags = ServerInstance->Config->ConfTags("wsorigin");
-		if (tags.first == tags.second)
-			throw ModuleException("You have loaded the websocket module but not configured any allowed origins!");
+		auto tags = ServerInstance->Config->ConfTags("wsorigin");
+		if (tags.empty())
+			throw ModuleException(this, "You have loaded the websocket module but not configured any allowed origins!");
 
 		WebSocketConfig config;
-		for (ConfigIter i = tags.first; i != tags.second; ++i)
-		{
-			ConfigTag* tag = i->second;
 
+		// If a server has a non-utf8 compatible character set configured
+		// then we can not support text encoding.
+		ISupport::TokenMap tokens;
+		isupportprov.Call(&ISupport::EventListener::OnBuildISupport, tokens);
+		auto it = tokens.find("CHARSET");
+		config.allowtext = it == tokens.end()
+			|| irc::equals(it->second, "ascii")
+			|| irc::equals(it->second, "utf8");
+
+		for (const auto& [_, tag] : tags)
+		{
 			// Ensure that we have the <wsorigin:allow> parameter.
 			const std::string allow = tag->getString("allow");
 			if (allow.empty())
-				throw ModuleException("<wsorigin:allow> is a mandatory field, at " + tag->getTagLocation());
+				throw ModuleException(this, "<wsorigin:allow> is a mandatory field, at " + tag->source.str());
 
 			config.allowedorigins.push_back(allow);
 		}
 
-		ConfigTag* tag = ServerInstance->Config->ConfValue("websocket");
+		const auto& tag = ServerInstance->Config->ConfValue("websocket");
 
-		const std::string defaultmodestr = tag->getString("defaultmode", tag->getBool("sendastext", true) ? "text" : "binary", 1);
-		if (stdalgo::string::equalsci(defaultmodestr, "reject"))
+		const std::string defaultmodestr = tag->getString("defaultmode", tag->getBool("sendastext", config.allowtext) ? "text" : "binary", 1);
+		if (insp::equalsci(defaultmodestr, "reject"))
 			config.defaultmode = WebSocketConfig::DM_REJECT;
-		else if (stdalgo::string::equalsci(defaultmodestr, "binary"))
+		else if (insp::equalsci(defaultmodestr, "binary"))
 			config.defaultmode = WebSocketConfig::DM_BINARY;
-		else if (stdalgo::string::equalsci(defaultmodestr, "text"))
+		else if (insp::equalsci(defaultmodestr, "text"))
 			config.defaultmode = WebSocketConfig::DM_TEXT;
 		else
-			throw ModuleException(defaultmodestr + " is an invalid value for <websocket:defaultmode>; acceptable values are 'binary', 'text' and 'reject', at " + tag->getTagLocation());
+			throw ModuleException(this, defaultmodestr + " is an invalid value for <websocket:defaultmode>; acceptable values are 'binary', 'text' and 'reject', at " + tag->source.str());
+
+		if (config.defaultmode == WebSocketConfig::DM_TEXT && !config.allowtext)
+			throw ModuleException(this, "You can not use text websockets when using a non-utf8 compatible server charset, at " + tag->source.str());
 
 		irc::spacesepstream proxyranges(tag->getString("proxyranges"));
 		for (std::string proxyrange; proxyranges.GetToken(proxyrange); )
 			config.proxyranges.push_back(proxyrange);
 
+		config.allowmissingorigin = tag->getBool("allowmissingorigin", true);
+		config.nativeping = tag->getBool("nativeping", true);
+
 		// Everything is okay; apply the new config.
 		hookprov->config = config;
 	}
 
-	void OnCleanup(ExtensionItem::ExtensibleType type, Extensible* item) CXX11_OVERRIDE
+	void OnCleanup(ExtensionType type, Extensible* item) override
 	{
-		if (type != ExtensionItem::EXT_USER)
+		if (type != ExtensionType::USER)
 			return;
 
 		LocalUser* user = IS_LOCAL(static_cast<User*>(item));
@@ -661,9 +758,26 @@ class ModuleWebSocket : public Module
 			ServerInstance->Users.QuitUser(user, "WebSocket module unloading");
 	}
 
-	Version GetVersion() CXX11_OVERRIDE
+	void OnWhois(Whois::Context& whois) override
 	{
-		return Version("Allows WebSocket clients to connect to the IRC server.", VF_VENDOR);
+		if (!whois.GetSource()->HasPrivPermission("users/auspex"))
+			return; // Only visible to opers.
+
+		const auto* origin = data.origin.Get(whois.GetTarget());
+		const auto* realhost = data.realhost.Get(whois.GetTarget());
+		const auto* realip = data.realip.Get(whois.GetTarget());
+		if (!origin && !realhost && !realip)
+			return; // If these fields are not set then the client is not a WebSocket user.
+
+		// If either of these aren't set then the user's connection isn't proxied.
+		std::string missing = "*";
+		if (!realhost || !realip)
+			realhost = realip = &missing;
+
+		if (origin)
+			whois.SendLine(RPL_WHOISGATEWAY, *realhost, *realip, "is connected from the " + *origin + " WebSocket origin");
+		else
+			whois.SendLine(RPL_WHOISGATEWAY, *realhost, *realip, "is connected from a non-web WebSocket origin");
 	}
 };
 
